#-*- coding: utf-8 -*- 
"""
  flatcms : A simple CMS for Django
  (C)2010 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt

  FkFilterSpec by Luc JEAN - http://djangosnippets.org/snippets/2194/
  AlphabeticFilterSpec by Marinho Brandao <marinho at gmail.com>
  see http://djangosnippets.org/snippets/1051/
"""

from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.conf import settings
from django.db import models
from django.contrib.admin.filterspecs import FilterSpec, ChoicesFilterSpec
from django.utils.encoding import smart_unicode
from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse

class FkFilterSpec(ChoicesFilterSpec):
    
    def __init__(self, f, request, params, model, model_admin):
        super(FkFilterSpec, self).__init__(f, request, params, model, model_admin)
        
        # ******* Extract parameters ********
        the_args = f.fk_filterspec.copy() 
        #The field of the related table
        fk_field = the_args['fk_field'] 
        
        #The name in the related table to use as label in the choices
        label = the_args.pop('label', '')
        
        #a title: by default the lookup arg
        self.filter_title = the_args.pop('title', '')
        
        #the foreign key field. By default the field the filter is assigned
        fk = the_args.pop('fk', f.name) 
        
        # ******* Build the filter definition ********
        
        self.lookup_kwarg = '{0}__{1}'.format(fk, fk_field)
        self.lookup_val = request.GET.get(self.lookup_kwarg, None)
        self.lookup_labels = {}
        #get the list of values
        values_list = model.objects.values_list(self.lookup_kwarg, flat=True)
        #get the 
        if label:
            label_field = '{0}__{1}__{2}'.format(fk, fk_field, label)
        else:
            label_field = '{0}__{1}'.format(fk, fk_field)
        labels = model.objects.values_list(label_field, flat=True)
        for (v, l) in zip(values_list, labels):
            self.lookup_labels[v] = l
        self.lookup_choices = self.lookup_labels.keys()

    def choices(self, cl):
        yield {'selected': self.lookup_val is None,
                'query_string': cl.get_query_string({}, [self.lookup_kwarg]),
                'display': _('All')}
        for val in self.lookup_choices:
            yield {'selected': smart_unicode(val) == self.lookup_val,
                    'query_string': cl.get_query_string({self.lookup_kwarg: val}),
                    'display': smart_unicode(self.lookup_labels[val])}
    
    def title(self):
        if self.filter_title:
            return self.filter_title
        else:
            return super(FkFilterSpec, self).title()

    @classmethod
    def register_filterspec(cls):
        """register the filter. To be called in the models.py"""
        FilterSpec.filter_specs.insert(0,
            (lambda f: len(getattr(f, 'fk_filterspec', [])), cls)
        )

class AlphabeticFilterSpec(ChoicesFilterSpec):
    """
    # Authors: Marinho Brandao <marinho at gmail.com>
    #          Guilherme M. Gondim (semente) <semente at taurinus.org>
    # File: <your project>/admin/filterspecs.py

    Adds filtering by first char (alphabetic style) of values in the admin
    filter sidebar. Set the alphabetic filter in the model field attribute
    'alphabetic_filter'.

    my_model_field.alphabetic_filter = True
    """

    def __init__(self, f, request, params, model, model_admin):
        super(AlphabeticFilterSpec, self).__init__(f, request, params, model,
                                                   model_admin)
        self.lookup_kwarg = '%s__istartswith' % f.name
        self.lookup_val = request.GET.get(self.lookup_kwarg, None)
        values_list = model.objects.values_list(f.name, flat=True)
        # getting the first char of values
        self.lookup_choices = list(set(val.upper()[0] for val in values_list if val))
        self.lookup_choices.sort()

    def choices(self, cl):
        yield {'selected': self.lookup_val is None,
                'query_string': cl.get_query_string({}, [self.lookup_kwarg]),
                'display': _('All')}
        for val in self.lookup_choices:
            yield {'selected': smart_unicode(val) == self.lookup_val,
                    'query_string': cl.get_query_string({self.lookup_kwarg: val}),
                    'display': val.upper()}
    def title(self):
        return _('%(field_name)s that starts with') % \
            {'field_name': self.field.verbose_name}

    @classmethod
    def register_filterspec(cls):
        """register the filter. To be called in the models.py"""
        FilterSpec.filter_specs.insert(0,
            (lambda f: getattr(f, 'alphabetic_filter', False), cls)
        )

class FileExtensionFilterSpec(ChoicesFilterSpec):
    """
    """

    def __init__(self, f, request, params, model, model_admin):
        super(FileExtensionFilterSpec, self).__init__(f, request, params, model,
                                                   model_admin)
        self.lookup_kwarg = '%s__iendswith' % f.name
        self.lookup_val = request.GET.get(self.lookup_kwarg, None)
        values_list = model.objects.values_list(f.name, flat=True)
        # getting the extension
        import os.path
        self.lookup_choices = list(set(os.path.splitext(val)[1].lower() for val in values_list if val))
        self.lookup_choices.sort()

    def choices(self, cl):
        yield {'selected': self.lookup_val is None,
                'query_string': cl.get_query_string({}, [self.lookup_kwarg]),
                'display': _('All')}
        for val in self.lookup_choices:
            yield {'selected': smart_unicode(val) == self.lookup_val,
                    'query_string': cl.get_query_string({self.lookup_kwarg: val}),
                    'display': val}
    def title(self):
        return _('file extension')

    @classmethod
    def register_filterspec(cls):
        """register the filter. To be called in the models.py"""
        FilterSpec.filter_specs.insert(0,
            (lambda f: getattr(f, 'file_extenion_filter', False), cls)
        )
