# -*- coding: utf-8 -*-
"""
  flatcms : A simple CMS for Django
  (C)2010 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt
  This is very influenced by the Django Flatpage subframework
"""

from views import get_cms_page, get_404, get_403
from django.http import Http404
from settings import MULTI_LINGUAL
from django.conf import settings
from models import CmsPage, SiteConfig, Alias, DjangoPage
from django.contrib.sites.models import Site
from django.utils import translation
from django.core.urlresolvers import reverse
from utils import get_django_page

def is_language_dependant(request):
    if MULTI_LINGUAL:
        #No multilingual for media and admin pages
        for prefix in (settings.MEDIA_URL, reverse('admin:index')):
            if request.path.startswith(prefix):
                return False
    return MULTI_LINGUAL

class FlatcmsFallbackMiddleware(object):
    
    def process_request(self, request):
        if is_language_dependant(request):
            site = Site.objects.get_current()
            try:
                page = CmsPage.objects.get(url=request.path, site=site)
                config = page.get_config()
            except CmsPage.DoesNotExist:
                try:
                    alias = Alias.objects.get(url=request.path, site=site)
                    config = alias.get_config()
                except Alias.DoesNotExist:
                    try:
                        page = get_django_page(site, request.path)
                        if page:
                            config = page.config
                        else:
                            config = SiteConfig.objects.get(site=site).default_config
                    except DjangoPage.DoesNotExist:
                        config = SiteConfig.objects.get(site=site).default_config
            
            if config.lang and translation.check_for_language(config.lang):
                translation.activate(config.lang)
                request.LANGUAGE_CODE = translation.get_language()
    
    def process_response(self, request, response):
        if response.status_code == 403:
            response = get_403(request)
            
        if response.status_code == 404:
            try:
                response = get_cms_page(request, request.path, middleware=True)
            # Return the original response if any errors happened. Because this
            # is a middleware, we can't assume the errors will be caught elsewhere.
            except Http404:
                response = get_404(request)
            except Exception, e:
                if settings.DEBUG:
                    print 'ERROR', e
        if is_language_dependant(request):
            translation.deactivate()
        return response
