# -*- coding: utf-8 -*-
"""
  flatcms : A simple CMS for Django
  (C)2010 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt
"""

from django.db import models
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as __
from django.conf import settings as project_settings
import settings
from django.forms.fields import RegexField
import datetime, re
from django.utils.safestring import mark_safe
from django.db.models.signals import pre_save
#from filterspecs import AlphabeticFilterSpec, FileExtensionFilterSpec, FkFilterSpec

#AlphabeticFilterSpec.register_filterspec()
#FileExtensionFilterSpec.register_filterspec()
#FkFilterSpec.register_filterspec()

#Force the filename to be lowercase in order to avoid problems
#in file names under WIndows and Linux
def format_filename(instance, folder, filename):
    return u'{0}/{1}'.format(folder, filename.lower())

def format_image_filename(instance, filename):
    return format_filename(instance, 'images', filename)

def format_file_filename(instance, filename):
    return format_filename(instance, 'files', filename)

class CustomPage:
    def __init__(self, url, title, desc, config=None):
        self.url = url
        self.title = title
        self.description = desc
        self.published = True
        self.breadcrumb_text = ''
        self.breadcrumb_parent = None
        self.config = config

class PageUrlFormField(RegexField):
    """Validation of URLS"""
    default_error_messages = {
        'invalid': _(u"Enter a valid local URL"),
    }
    
    validation_regex = re.compile(r'^/[-\w/\.]*$')

    def __init__(self, *args, **kwargs):
        super(PageUrlFormField, self).__init__(PageUrlFormField.validation_regex, *args, **kwargs)

class PageUrlField(models.CharField):
    def __init__(self, verbose_name=None, name=None, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 200)
        models.CharField.__init__(self, verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'form_class': PageUrlFormField}
        defaults.update(kwargs)
        return super(PageUrlField, self).formfield(**defaults)

class MenuUrlFormField(RegexField):
    """Validation of URLS"""
    default_error_messages = {
        'invalid': _(u"Enter a valid URL"),
    }
    
    validation_regex = re.compile(r'^(https?://[\w\d\.\-]+|/)[\w/\.\-\?\=&\%\:]*$')

    def __init__(self, *args, **kwargs):
        super(MenuUrlFormField, self).__init__(MenuUrlFormField.validation_regex, *args, **kwargs)

class MenuUrlField(models.CharField):
    def __init__(self, verbose_name=None, name=None, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 200)
        models.CharField.__init__(self, verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'form_class': MenuUrlFormField}
        defaults.update(kwargs)
        return super(MenuUrlField, self).formfield(**defaults)

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["flatcms\.models\..*UrlField"])
except:
    pass

class SiteConfig(models.Model):
    
    ANALYTICS_OPTIONS = (
        ('disabled', _('Disabled')),
        ('single', _('One domain')),
        ('single-and-sub', _('One domain and subdomains')),
        ('multi', _('Several domains')),
    )
    
    """Defines the site look-and-feel: base template, logo, css..."""
    site = models.ForeignKey(Site, default=project_settings.SITE_ID, unique=True)
    name = models.CharField(_('Name'), max_length=100, blank=True, default="")
    logo = models.ImageField(_('Logo'), upload_to=format_image_filename, default='', blank=True)
    favicon = models.ImageField(_('Favicon'), upload_to=format_image_filename, default='', blank=True)
    designer = models.CharField(_('Designer'), max_length=100, default = "", blank=True)
    designer_website = models.URLField(_('Designer website'), max_length=100, default="", blank=True)
    design_year = models.CharField(_('Design Year'), max_length=10, default = datetime.date.today().strftime('%Y'))
    analytics_code = models.CharField(_('Google Analytics Code'), max_length=32, default = '', blank=True)
    analytics_option = models.CharField(_('Google Analytics Option'), max_length=32,
                                        default = 'disabled', choices=ANALYTICS_OPTIONS)
    show_login_banner = models.BooleanField(_('Display login on banner'), default=settings.USE_AS_EXTRANET)
    show_site_in_title = models.BooleanField(_('Show the site name in the page title'), default=True)
    custom_banner = models.BooleanField(_('Make possible to use plugins for the banner'), default=False)
    show_title_in_content = models.BooleanField(_('Show the title in content'), default=False)
    default_config = models.ForeignKey('flatcms.PageConfig',
        help_text=_("defines which config to use if nothing is defined for the page"))
    
    def __unicode__(self):
        return u"%s" % (self.name)
    
    class Meta:
        verbose_name = _("Site configuration")
        verbose_name_plural = _("Site configurations")
        

class Image(models.Model):
    """An image that can be used into a CmsPage"""
    class Meta:
        ordering = ['ordering']
        verbose_name = _("Image")
        verbose_name_plural = _("Images")
    
    name = models.CharField(_('Name'), max_length=128)
    image = models.ImageField(_('File'), upload_to=format_image_filename)
    image.file_extenion_filter = True
    ordering = models.IntegerField(_('Ordering'), default=10,
        help_text=_("used to sort images on a slideshow"))
    lastmod = models.DateTimeField(_('Last modification'), auto_now=True, default=datetime.datetime.now())
    
    def url(self):
        return u"{0}{1}".format(project_settings.MEDIA_URL, self.image)

    def __unicode__(self):
        return u"{0} -- {1}".format(self.name, self.image)

class Slideshow(models.Model):
    """Display images as a slideshow"""
    class Meta:
        verbose_name = _("Slideshow")
        verbose_name_plural = _("Slideshows")

    def EFFECTS_CHOICE():
        supported_effects = ("blindX", "blindY", "blindZ", "cover", "curtainX", "curtainY",
                "fade", "fadeZoom", "growX", "growY", "scrollUp", "scrollDown",
                "scrollLeft", "scrollRight", "scrollHorz", "scrollVert", "shuffle",
                "slideX", "slideY", "toss", "turnUp", "turnDown", "turnLeft",
                "turnRight", "uncover", "wipe", "zoom")
        choices = []
        for effect in supported_effects:
            choices.append((effect, _(effect)))
        return tuple(choices)
    
    name = models.CharField(_('Name'), max_length=64, unique=True)
    images = models.ManyToManyField(Image)
    width = models.IntegerField(_('Width'), default=200)
    height = models.IntegerField(_('Height'), default=200)
    effect = models.CharField(_('Effect'), max_length="32", choices=EFFECTS_CHOICE(), default="fade")
    speed = models.IntegerField(_('Speed'), default=5000)
    timeout = models.IntegerField(_('Timeout'), default=0)
    random = models.IntegerField(_('Random'), default=0)
    
    def __unicode__(self):
        return u"%s" % (self.name)

class MetaHeader(models.Model):
    """The meta placed in the http headers of the file"""
    class Meta:
        verbose_name = _("Meta header")
        verbose_name_plural = _("Meta headers")

    site_config = models.ForeignKey(SiteConfig)
    name = models.CharField(_('Name'), max_length=50)
    content = models.CharField(_('Content'), max_length=100)

    def __unicode__(self):
        return u'<meta name="%s" content="%s" />' % (self.name, self.content)
    
class CssSelector(models.Model):
    """A CSS selector ex : #main, #main a"""
    class Meta:
        ordering = ['ordering']
        verbose_name = _("CSS selector")
        verbose_name_plural = _("CSS selectors")

    site = models.ManyToManyField(Site)
    selector = models.CharField(_('Selector'), max_length=200)
    selector.alphabetic_filter = True
    enabled = models.BooleanField(_('Enabled'), default=True)
    ordering = models.IntegerField(_('Ordering'), default=10,
        help_text=_("menu are sorted with this number in ascending order"))
    in_editor = models.BooleanField(_('In editor'), default=False,
        help_text=_("If checked, the style is available in the CMS page editor"))
    
    def __unicode__(self):
        return u"%s" % (self.selector)

class CssProperty(models.Model):
    """a Css Property ex: 'color: black;' property='color' and value='black'"""
    selector = models.ForeignKey(CssSelector)
    property = models.CharField(_('Property'), max_length=100)
    value = models.CharField(_('Value'), max_length=200)

    class Meta:
        verbose_name = _("CSS Property")
        verbose_name_plural = _("CSS Properties")

    def __unicode__(self):
        return u'%s: %s;' % (self.property, self.value)

class HeaderInclude(models.Model):
    """A line to include in the html header"""
    class Meta:
        verbose_name = _("header include")
        verbose_name_plural = _("header includes")

    name = models.CharField(_('Name'), max_length=50)
    
    def __unicode__(self):
        args = []
        for arg in self.headerincludeargument_set.all():
            args.append(u'{0}'.format(arg))
        return u'<{0} {1} />'.format(self.name, u' '.join(args))
    
class HeaderIncludeArgument(models.Model):
    """Argument of a line to include in a html header"""
    class Meta:
        verbose_name = _("header include argument")
        verbose_name_plural = _("header include arguments")

    include = models.ForeignKey(HeaderInclude)
    name = models.CharField(_('Name'), max_length=50)
    value = models.CharField(_('Value'), max_length=100)
    
    def __unicode__(self):
        return u'{0}="{1}"'.format(self.name, self.value)


class Menu(models.Model):
    """A menu is a list of menu items"""
    class Meta:
        ordering = ['ordering']
        verbose_name = _("Menu")
        verbose_name_plural = _("Menus")
    
    label = models.CharField(_('Label'), max_length=32)
    div_id = models.CharField(_('div id'), max_length=32, blank=True, default='',
        choices=settings.DIV_ID_CHOICES, help_text=_("the identifier of the html div container"))
    ordering = models.IntegerField(_('ordering'), default=1,
        help_text=_("menu are sorted with this number in ascending order"))
    published = models.BooleanField(_('Published'), default=settings.PUBLISH_BY_DEFAULT,
        help_text=_("If this is not checked, it is not displayed on the page."))
    registration_required = models.BooleanField(_('Registration required'),
        help_text=_("If this is checked, only logged-in users will be able to view the page."))
    
    def __unicode__(self):
        return u"%s" % (self.label)
    
class MenuItem(models.Model):
    """The items displayed on the menu of a given page"""
    class Meta:
        ordering = ['ordering']
        verbose_name = _("Menu item")
        verbose_name_plural = _("Menu items")
        
    TARGET_CHOICES = (
        ('', _('This window')),
        ('target="_blank"', _('New window')),
        ('target="_parent"', _('parent window')),
        ('target="_top"', _('top window')),
    )

    parent_menu = models.ForeignKey(Menu, related_name='parent', verbose_name=_('parent menu'))
    label = models.CharField(_('Label'), max_length=32)
    url = MenuUrlField(_('URL'), max_length=100, blank=True)
    page = models.ForeignKey('flatcms.CmsPage', blank=True, null=True, default='')
    child_menu = models.ForeignKey(Menu, null=True, blank=True, default='',
                                   verbose_name=_('child menu'), related_name='child')
    target = models.CharField(_('Target'), max_length=100, blank=True, choices=TARGET_CHOICES, default='')
    ordering = models.IntegerField(_('Ordering'), default=1,
        help_text=_("menu items are sorted with this number in ascending order"))
    published = models.BooleanField(_('Published'), default=settings.PUBLISH_BY_DEFAULT,
        help_text=_("If this is not checked, it is not displayed on the page."))
    registration_required = models.BooleanField(_('registration required'),
        help_text=_("If this is checked, only logged-in users will be able to view the page."))
    py_code = models.CharField(_('Python code'), default='', max_length=100, blank=True,
        choices=settings.PYCODE_CHOICES, help_text=_("Content is generated thanks to a Python function"))
    force_selected = False
    
    def __unicode__(self):
        if self.page:
            return u"%s-%s" % (self.label, self.page.url)
        else:
            return u"%s-%s" % (self.label, self.url)
    
class Plugin(models.Model):
    """Some subcontent that can be added to a given page"""
    class Meta:
        ordering = ['ordering']
        verbose_name = _("Plugin")
        verbose_name_plural = _("Plugins")
    
    name = models.CharField(_('Name'), max_length=32, db_index=True)
    div_id = models.CharField(_('div id'), default='rightcol', max_length=32,
        choices=settings.DIV_ID_CHOICES, help_text=_("the identifier of the html div container"))
    published = models.BooleanField(_('Published'), default=settings.PUBLISH_BY_DEFAULT,
        help_text=_("If this is not checked, it is not displayed on the page."))
    ordering = models.IntegerField(_('Ordering'), default=1,
        help_text=_("plugins are sorted with this number in ascending order"))
    content = models.TextField(_("Content"), blank=True)
    registration_required = models.BooleanField(_('registration required'),
        help_text=_("If this is checked, only logged-in users will be able to view the page."))
    slideshow = models.ForeignKey(Slideshow, blank=True, null=True, default='', verbose_name=_("slideshow"))
    py_code = models.CharField(_('Python code'), default='', max_length=100, blank=True,
        choices=settings.PYCODE_CHOICES, help_text=_("Content is generated thanks to a Python function"))
    
    def __unicode__(self):
        return u"%s -- %s" % (self.name, self.div_id)

class StepBanner(models.Model):
    """
    This banner can be shown on the top of the page and
    show different steps to reach a goal   
    """
    name = models.CharField(_('Name'), max_length=32)
    show_num = models.BooleanField(default=False)
    show_all = models.BooleanField(default=False,
        help_text=_("Show all items even if the page level is below the item level"))

    class Meta:
        verbose_name = _("Step banner")
        verbose_name_plural = _("Step banners")
    
    def __unicode__(self):
        return self.name
    
class Step(models.Model):
    """"""
    label = models.CharField(_('Label'), max_length=32)
    url = MenuUrlField(_('URL'), max_length=100, blank=True)
    page = models.ForeignKey('flatcms.CmsPage', blank=True, null=True, default='')
    level = models.IntegerField(_('Level'))
    banner = models.ForeignKey(StepBanner)

    class Meta:
        verbose_name = _("Step")
        verbose_name_plural = _("Steps")

    def __unicode__(self):
        return u'{0}-{1}'.format(self.banner, self.label)

class PageConfig(models.Model):
    """Defines menus and plugins to be used in a page"""
    class Meta:
        verbose_name = _("Page configuration")
        verbose_name_plural = _("Page configurations")

    name = models.CharField(_('Name'), max_length=32, db_index=True)
    plugins = models.ManyToManyField(Plugin, blank=True, verbose_name=_("plugins"))
    menu = models.ManyToManyField(Menu, blank=True, verbose_name=_("menus"))
    template_name = models.CharField(_('template'), max_length=70, blank=True, choices=(('',''),)+settings.TEMPLATE_CHOICES,
        help_text=_("defines which template to use for the page. If blank use the site default template"), default='')
    step_banner = models.ForeignKey(StepBanner, verbose_name=_("StepBanner"), blank=True, default='', null=True)
    header_includes = models.ManyToManyField(HeaderInclude, blank=True, verbose_name=_("header includes"))
    lang = models.CharField(_('Language'), max_length=10, default='', blank=True, choices=settings.SUPPORTED_LANGUAGES)
    
    def __unicode__(self):
        return u"%s" % (self.name)

class PageOption(models.Model):
    """"""
    OPTION_TYPE_STRING = 1
    OPTION_TYPE_NUMERIC = 2
    OPTION_TYPE_CHOICES = (
        (OPTION_TYPE_STRING, _('String')),
        (OPTION_TYPE_NUMERIC, _('Numeric')),
    )
        
    config = models.ForeignKey(PageConfig)
    name = models.CharField(_('Name'), max_length=64, choices=settings.OPTION_CHOICES)
    value = models.CharField(_('Value'), max_length=100)
    type = models.IntegerField(_('Type'), default=OPTION_TYPE_STRING, choices=OPTION_TYPE_CHOICES)

    class Meta:
        verbose_name = _("Page option")
        verbose_name_plural = _("Page options")
        
    def typed_value(self):
        if self.type == self.OPTION_TYPE_STRING:
            return mark_safe(u"'{0}'".format(self.value))
        return self.value

    def __unicode__(self):
        return u'{0}={1}'.format(self.name, self.typed_value())

class CmsPage(models.Model):
    """A CMS page : very influenced by the django FlatPage"""
    CHANGEFREQ_CHOICES = (
        ('always', _('always')),
        ('hourly', _('hourly')),
        ('daily', _('daily')),
        ('weekly', _('weekly')),
        ('monthly', _('monthly')),
        ('never', _('never')),
    )

    class Meta:
        verbose_name = _("CMS Page")
        verbose_name_plural = _("CMS Pages")
        ordering = ['url']
        unique_together = (('url', 'site'),)

    url = PageUrlField(_('URL'), max_length=100, db_index=True)
    title = models.CharField(_('Title'), max_length=200)
    description = models.CharField(_('Description'), max_length=200, blank=True)
    registration_required = models.BooleanField(_('Registration required'),
        help_text=_("If this is checked, only logged-in users will be able to view the page."),
        default=settings.USE_AS_EXTRANET)
    site = models.ForeignKey(Site, default=project_settings.SITE_ID)
    content = models.TextField(_('Content'), blank=True)
    published = models.BooleanField(_('Published'), default=settings.PUBLISH_BY_DEFAULT,
        help_text=_("If this is checked, the page will be published on the site."))
    config = models.ForeignKey(PageConfig, verbose_name=_("Configuration"), default='', blank=True, null=True)
    sitemap = models.BooleanField(_('Sitemap'), help_text=_("Included in the sitemap.xml"), default=True) 
    changefreq = models.CharField(_('Change frequency'), max_length=10, choices=CHANGEFREQ_CHOICES, default='never')
    lastmod = models.DateTimeField(_('Last modification'), auto_now=True, default=datetime.datetime.now())
    priority = models.DecimalField(_("Priority"), max_digits=2, decimal_places=1, default="0.5")
    step_level = models.IntegerField(_('Step level'), default=0)
    app_only = models.BooleanField(_('App only'), default=False,
        help_text=_("If check, the page can only be used by a Django application. Keep No in case of doubt"))
    breadcrumb_text = models.CharField(_('breadcrumb text'), max_length=100, blank=True, default="")
    breadcrumb_parent = models.ForeignKey('flatcms.CmsPage', blank=True, null=True, default="",
        verbose_name = _('breadcrumb parent'))
    show_in_rss = models.BooleanField(_('Rss feed'), default=settings.SHOW_IN_RSS)
    
    url.fk_filterspec = {'fk_field':'lang', 'fk':'config', 'title':_("Language")}

    def __unicode__(self):
        return u"{0} - {1}".format(self.url, self.site)

    def get_config(self):
        return self.config or SiteConfig.objects.get(site=self.site).default_config

    def get_absolute_url(self):
        return self.url
    
    def full_url(self):
        return u'http://{0}{1}'.format(self.site, self.url)

    @staticmethod
    def update_links(**kwargs):
        instance = kwargs.get('instance')
        
        if instance.id: #for existing pages
            try:
                old_url = CmsPage.objects.get(id=instance.id).url
            except CmsPage.DoesNotExist:
                #This may occur when loading json file
                return
                
            if old_url != instance.url: #If the url has changed
                #replace in current page
                if instance.content.find(old_url)>=0:
                    instance.content = instance.content.replace(old_url, instance.url)
                    instance.save()
                
                #replace the old url with the new one in all pages
                for page in CmsPage.objects.filter(content__contains=old_url).exclude(id=instance.id): 
                    page.content = page.content.replace(old_url, instance.url)
                    page.save()
                        
                #and plugins
                for plugin in Plugin.objects.filter(content__contains=old_url):
                    plugin.content = plugin.content.replace(old_url, instance.url)
                    plugin.save()
                
    

class Alias(models.Model):
    """an alias is an Url that will point to an existing page"""
    url = PageUrlField(_('URL'), max_length=100, db_index=True)
    site = models.ForeignKey(Site, default=project_settings.SITE_ID)
    page = models.ForeignKey(CmsPage)
    config = models.ForeignKey(PageConfig, verbose_name=_("Configuration"),
                               blank=True, null=True, default='')
    redirect = models.BooleanField(verbose_name=_("Redirect"), default=False)
    
    sitemap = models.BooleanField(_('Sitemap'), help_text=_("Included in the sitemap.xml"), default=True) 

    class Meta:
        verbose_name = _("Alias")
        verbose_name_plural = _("Aliases")
        unique_together = (('url', 'site'),)

    def __unicode__(self):
        return u"{0} - {1}".format(self.url, self.site)
        
    def get_config(self):
        return self.config or self.page.config or SiteConfig.objects.get(site=self.site).default_config

    def get_absolute_url(self):
        return self.url
    
    def full_url(self):
        return u'http://{0}{1}'.format(self.site, self.url)
    

class Download(models.Model):
    """a file that can be downloaded"""
    file = models.FileField(_('File'), upload_to=format_file_filename)
    file.file_extenion_filter = True   
    class Meta:
        verbose_name = _("File")
        verbose_name_plural = _("Files")

    def __unicode__(self):
        return unicode(self.file)


class DjangoPage(models.Model):
    class Meta:
        verbose_name = _("django page")
        verbose_name_plural = _("django pages")
        ordering = ['ordering']

    regex = models.CharField(_('regular expression'), max_length=100, db_index=True)
    title = models.CharField(_('Title'), max_length=200)
    description = models.CharField(_('Description'), max_length=200, blank=True)
    site = models.ForeignKey(Site, default=project_settings.SITE_ID)
    config = models.ForeignKey(PageConfig, verbose_name=_("Configuration"), default='', blank=True, null=True)
    ordering = models.IntegerField(_("Ordering"), default=1)
    
    def get_config(self):
        return self.config or SiteConfig.objects.get(site=self.site).default_config

    def __unicode__(self):
        return u"{0} - {1}".format(self.regex, self.config.name)

pre_save.connect(CmsPage.update_links, CmsPage)