# -*- coding: utf-8 -*-
"""
  flatcms : A simple CMS for Django
  (C)2010 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt
"""

from django.conf import settings as django_settings
import glob, os.path, sys, datetime, re
from django.utils.translation import ugettext as _

class TemplatesFinder:
    """Build the list of templates that must be displayed as base template for a CmsPage"""
    
    def _find(self, dirName, files):
        """add the templates ending by '_.html' to the list of base template for CmsPage"""
        dirNamePrefix = dirName+"/*"
        for filename in glob.glob(dirNamePrefix):
            if os.path.isfile(filename):
                if filename[-6:]=="_.html":
                    name = filename[len(self.folder)+1:][:-6]
                    name = name.replace('/', '.').replace('\\', '.')
                    self.templates.append((name, name))
                    
                
    def _find_all(self, folder):
        """Start the search of templates"""
        self.folder = folder
        os.path.walk(self.folder, TemplatesFinder._find, self)

    def get_templates(self):
        """return the list of tempaltes. Search into the TEMPLATE_DIRS"""
        self.templates = []
        flatcms_default_templates_dir = os.path.normpath(os.path.dirname(os.path.abspath(__file__))+'/templates')
        lookup_folders = list(set([d for d in django_settings.TEMPLATE_DIRS]).union([flatcms_default_templates_dir]))
        for folder in lookup_folders:
            self._find_all(folder)
        return tuple(self.templates)

#If True: the CmsPage.registration_required will be True by default
USE_AS_EXTRANET = getattr(django_settings, 'FLATCMS_USE_AS_EXTRANET', False)

#If True: The CmsPage.show_in_rss will be True by default
SHOW_IN_RSS = not USE_AS_EXTRANET

#If True: The CmsPage.publisehd will be True by default
PUBLISH_BY_DEFAULT = not getattr(django_settings, 'FLATCMS_MODERATOR_MODE', False)

#If True, the CMS is in multilingual mode
#A language must be specified for each page
#When the page is displayed, the locale is changed
MULTI_LINGUAL = getattr(django_settings, 'FLATCMS_MULTILINGUAL', False)
SUPPORTED_LANGUAGES = django_settings.LANGUAGES if MULTI_LINGUAL else (('', ''),)

#The list of templates that can be used: combobox in the admin
#If missing, browse the template folders to find templates ending with an '_'
TEMPLATE_CHOICES = getattr(django_settings, 'FLATCMS_TEMPLATES_CHOICE', TemplatesFinder().get_templates())

#The list of divid that can be used: combobox in the admin
DIV_ID_CHOICES = getattr(django_settings, 'FLATCMS_DIV_ID_CHOICES', None)

#The list of options that can be used
OPTION_CHOICES = getattr(django_settings, 'FLATCMS_OPTION_CHOICE', None)

#The list of Python function that can be called as py_code
#The functions must be listed here to be callable
PYCODE_CHOICES = getattr(django_settings, 'FLATCMS_PYCODE_CHOICES', None)

#The template to be used if nothing is defined
DEFAULT_TEMPLATE = getattr(django_settings, 'FLATCMS_DEFAULT_TEMPLATE', 'flatcms.default')

ROBOTS_DISALLOW = getattr(django_settings, 'FLATCMS_ROBOTS_DISALLOW', ("/c/",))