# -*- coding: utf-8 -*-
"""
  flatcms : A simple CMS for Django
  (C)2010 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt
"""

from django import template
from django.template.loader import get_template
from django.conf import settings
from flatcms.models import MenuItem
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from flatcms.utils import exec_menu_pycode

def is_object_visible(obj, user):
   """check if the current object (plugin, menu) must be inserted in the div"""
   return (obj.published or user.is_staff) and (not obj.registration_required or user.is_authenticated())

def tagify(text, tag, css_class="", css_id=""):
   """encapsulate the text in an HTML tag"""
   if type(css_class) is list:
      css_classes = u' class="{0}"'.format(' '.join(css_class))
   else:
      css_classes = u' class="{0}"'.format(css_class) if css_class else ''
   
   css_id = u' id="{0}"'.format(css_id) if css_id else ''
   return u'<{0}{2}{3}>{1}</{0}>'.format(tag, text, css_classes, css_id)

register = template.Library()

class FlatCmsPluginNode(template.Node):
   def __init__(self, plugins, div_id):
      self.plugins = template.Variable(plugins)
      self.div_id = div_id
      
   def get_content(self, plugin):
      slideshow = u''
      if plugin.slideshow:
         slideshow = u'<div class="flatcms-slideshow" id="flatcms-slideshow-%s">' % plugin.slideshow.name
         for image in plugin.slideshow.images.all():
            url = image.url()
            slideshow += u'<img src="{0}" width="{1}" height="{2}" />'.format(url, plugin.slideshow.width, plugin.slideshow.height)
         slideshow += u'</div>'
      return plugin.content + slideshow
      
   def render(self, context):
      try:
         plugins = self.plugins.resolve(context)
         user = template.Variable("user").resolve(context)
         
         html = ''
         
         for p in plugins:
            if (p.div_id==self.div_id) and is_object_visible(p, user):
               html += '<div class="flatcms-plugin" id="flatcms-plugin-%s">' % p.name
               if user.is_staff:
                  html += u'<span class="edit-plugin">{2} : <a href="{0}" target="_blank">{1}</a></span>'.format(
                     reverse('admin:flatcms_plugin_change', args=[p.pk]), _('Edit'), p.name)
               if not p.published:
                  html += '<div class="draft">{0}</div>'.format(self.get_content(p))
               else:
                  html += self.get_content(p)
               html += '</div>'
               
         return html
      
      except template.VariableDoesNotExist:
         return ''

@register.tag('flatcms-plugin')
def insert_plugin(parser, token):
   """
   """
   try:
      name, plugins, div_id = token.split_contents()
   except ValueError:
      raise template.TemplateSyntaxError, "%r tag requires exactly 2 arguments" % token.contents.split()
   
   return FlatCmsPluginNode(plugins, div_id)

class FlatCmsMenuNode(template.Node):
   def __init__(self, menus, div_id, css_class):
      self.menus = template.Variable(menus)
      self.div_id = div_id
      if css_class[:3]=='id=':
         self.css_id = css_class[3:]
         self.css_class = ""
      else:
         self.css_class = css_class
         self.css_id = ""
            
   def insert_item(self, item, page):
      """Add HTML code for a menu item: <a href="url" target>label</a>"""
      url = item.page.url if item.page else item.url 
      menu_item = u'<a href="{0}" {1} {2}>{3}</a>'.format(url, item.target,
                  u'' if item.published else u'class="draft"', item.label)
      if item.force_selected or (item.page and (item.page.url == page.url)) or (item.url == page.url):
         menu_item = tagify(menu_item, 'span', 'selmenuitem')
      return menu_item
            
   def insert_menu(self, menu, user, page, top=True):
      """Add HTML code for a menu: <ul><li>item1</li>...</ul>"""
      if is_object_visible(menu, user):
         items_html = self.insert_menu_items(menu, user, page)
         menu_html = tagify(items_html, u'ul', self.css_class if top else "",
                            self.css_id if top else "")
         if not menu.published:
            menu_html = tagify(menu_html, u'span', u'draft')
         return menu_html
      return ''

   def insert_menu_items(self, menu, user, page):
      """insert code for the given menu_item: dropdown menu or single item"""
      html = ''
      counter = 0
      for item in MenuItem.objects.select_related('child_menu', 'page').filter(parent_menu=menu):
         exec_menu_pycode(user, item)
         if is_object_visible(item, user):
            if item.child_menu:
               css_classes = [u'dropdown']
               if not item.published: css_classes += [u'draft']
               menu_item = tagify(item.label, u'a', css_classes)
               menu_item += self.insert_menu(item.child_menu, user, page, top=False)
         
            else:
               menu_item = self.insert_item(item, page)
               
            counter += 1
            html += tagify(menu_item, u'li', css_class='menuitem-{0}'.format(counter))
              
      return html

   def render(self, context):
      """return the HTML code corresponding to the templatetag"""
      try:
         menus = self.menus.resolve(context)
         user = template.Variable("user").resolve(context)
         page = template.Variable("page").resolve(context)
      
         html = ''
         for m in menus:
            if (m.div_id==self.div_id):
               if user.is_staff:
                  html += u'<span class="edit-plugin">{2} : <a href="{0}" target="_blank">{1}</a></span>'.format(
                     reverse('admin:flatcms_menu_change', args=[m.pk]), _('Edit'), m)
               html += self.insert_menu(m, user, page)
               
         return html
      
      except template.VariableDoesNotExist, msg:
         return ''

@register.tag('flatcms-menu')
def insert_menu(parser, token):
   """insert the menu"""
   try:
      name, menus, div_id, css_class = token.split_contents()
   except:
      try:
         css_class = ""
         name, menus, div_id = token.split_contents()
      except ValueError:
         raise template.TemplateSyntaxError, "%r tag requires exactly 2 arguments" % token.contents.split()
   
   return FlatCmsMenuNode(menus, div_id, css_class)

@register.filter
def page_breadcrumbs(page):
   breadcrumbs = []
   already_used = []
   cur_page = page
   
   while cur_page:
      if cur_page.breadcrumb_text:
         breadcrumbs.insert(0, {'text':cur_page.breadcrumb_text, 'url':cur_page.url})
      
      if cur_page.url in already_used:
         cur_page = None #Avoid dead loops
      else:
         already_used.append(cur_page.url)
         cur_page = cur_page.breadcrumb_parent
   
   return breadcrumbs 