# -*- coding: utf-8 -*-
"""
  flatcms : A simple CMS for Django
  (C)2010 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt
"""

from django.test import TestCase
from django.conf import settings as project_settings
import os, os.path, shutil
try:
    project_settings.UNIT_TEST = True
    project_settings.FLATCMS_TEMPLATES_CHOICE = None
except:
    pass
import models, views
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
import datetime
from django.db.utils import IntegrityError
from BeautifulSoup import BeautifulSoup
from django.utils.translation import ugettext as _
from django.core.files import File
import settings
from django.core import mail
import re

DRAFT = 'class="draft"'
SELECTED = 'class="selmenuitem"'
PWD = "luc"
STAFF_USER = "luc"
STD_USER = "bob"
COPYRIGHT_MARK = '<div id="copyright">'

#menu type
MENUITEM_URL = 0
MENUITEM_CHILD = 1
MENUITEM_PAGE = 2

class TemplatesFinderTest(TestCase):
    
    def find_templates(self, template_dir, flat_templates, base_name):
        """recursive calls to explore the template folders"""
        for item in os.listdir("{0}{1}{2}".format(template_dir, os.path.sep, base_name)):
            item_name = "{0}{1}{2}".format(base_name, os.path.sep, item) if base_name else item
            if not os.path.isdir("{0}{1}{2}".format(template_dir, os.path.sep, item_name)):
                if item_name[-6:] == '_.html':
                    flat_templates.append(item_name)
            else:
                self.find_templates(template_dir, flat_templates, item_name)
    
    def get_templates(self):
        """load the templates"""
        flat_templates = []
        flatcms_default_templates_dir = os.path.normpath(os.path.dirname(os.path.abspath(__file__))+'/templates')
        for template_dir in ([x for x in project_settings.TEMPLATE_DIRS]+[flatcms_default_templates_dir]):
            self.find_templates(template_dir, flat_templates, "")
        return tuple(flat_templates)
    
    def test_load_templates(self):
        """Tests the templates loaded with two distinct mechanism"""
        test_templates = [file.replace(os.path.sep, '.')[:-6] for file in self.get_templates()]
        templates = [file for (file, name) in settings.TemplatesFinder().get_templates()]
        self.assertEqual(set(templates), set(test_templates))

    def test_templates_choice(self):
        """Tests the templates loaded with two distinct mechanism"""
        for template in settings.TemplatesFinder().get_templates():
            self.assertEqual(type(template), tuple)
            self.assertEqual(len(template), 2)

def make_local(url):
    prefix = '/'+project_settings.LANGUAGE_CODE+'/'
    if url.find(prefix) != 0:
        return prefix[:-1]+url
    return url

class PageTest(TestCase):
    fixtures = ['user-unittest.json']
    
    def setUp(self):
        self._project_default_template = getattr(project_settings, 'FLATCMS_DEFAULT_TEMPLATE', None)
        self._pycode_choices = getattr(project_settings, 'FLATCMS_PYCODE_CHOICES', None)
        project_settings.FLATCMS_DEFAULT_TEMPLATE = 'flatcms.unittest'
        project_settings.FLATCMS_PYCODE_CHOICES = (('flatcms.tests.dummy_pycode', u'Dummy'),)
        reload(settings)
        
    def tearDown(self):
        project_settings.FLATCMS_DEFAULT_TEMPLATE = self._project_default_template
        project_settings.FLATCMS_PYCODE_CHOICES = self._pycode_choices
        
    def _create_site_config(self, name, site=None):
        site_config = models.SiteConfig()
        site_config.name = name
        config = models.PageConfig()
        config.name = name+"Config"
        config.save()
        if not site:
            site = Site.objects.get(id=project_settings.SITE_ID)
        site_config.site = site
        site_config.default_config = config
        site_config.save()
        return site_config
        
    def _create_page(self, url, title, content):
        if 'localeurl' in project_settings.INSTALLED_APPS:
            url = make_local(url)
        
        if models.SiteConfig.objects.count()==0:
            site_config = self._create_site_config('mysite')
    
        config = models.PageConfig()
        config.name = "Main"
        config.save()
        
        page = models.CmsPage()
        page.url = url
        page.title = title
        page.content = content
        page.config = config
        page.site = Site.objects.get(id=project_settings.SITE_ID)
        page.save()
        return page

    def _check_page(self, page, draft=False, err_code=0):
        response = self.client.get(page.url, follow=True)
        
        if err_code:
            self.assertEqual(response.status_code, err_code)
        
        #if response.status_code == 404:
        #    self.assertContains(response, page.title)
        #    self.assertContains(response, page.content)
        #    if draft:
        #        self.assertContains(response, DRAFT)
        #    else:
        #        self.assertNotContains(response, DRAFT)
        
        return response

    def _check_page_error(self, page, err_code, follow=True):
        response = self.client.get(page.url, follow=follow)
        self.failUnlessEqual(response.status_code, err_code)
        
    def _extract_tag(self, text, search, level=1):
        pos = text.find(search)
        if pos>0:
            reverse_text = text[:pos][::-1]
            x = 0
            for i in xrange(level):
                x = reverse_text.find("<", x+1)
                if x<0:
                    return ""
            start_pos = pos - x
            end_pos = text.find(">", start_pos)
            return text[start_pos:end_pos]
        return ""
        
    def _check_menu(self, response, label, url, draft=False, visible=True):
        if visible:
            self._check_sfmenu(response)
            self.assertContains(response, '++'+label+'++')
            self.assertContains(response, url)
            if draft != None:
                if draft:
                    self.assertContains(response, DRAFT)
                else:
                    self.assertNotContains(response, DRAFT)
        else:
            self.assertNotContains(response, label)
            self.assertNotContains(response, url)

    def _check_tag(self, response, content, contains, exists=True):
        tag = self._extract_tag(response.content, content)
        if exists:
            self.assertTrue(tag.find(contains)>=0)
        else:
            self.assertFalse(tag.find(contains)>=0)
            
    def _check_menu_item_is_draft(self, response, label, exists):
        #self._check_tag(response, '++'+label+'++', DRAFT, exists)
        tag = self._extract_tag(response.content, '++'+label+'++')
        match = re.search('class=".*draft.*"', tag)
        if exists:
            self.assertTrue(match)
        else:
            self.assertFalse(match)
        
    def _get_menu_item_label(self, label, menu_type):
        return '..'+label+'..' if MENUITEM_CHILD == menu_type else '++'+label+'++'

    def _check_menu_item_target(self, response, label, menu_type, exists, target="_blank"):
        self._check_tag(response, self._get_menu_item_label(label, menu_type), 'target="{0}"'.format(target), exists)

    def _check_menu_item_is_selected(self, response, label, menu_type, ok):
        tag = self._extract_tag(str(response), self._get_menu_item_label(label, menu_type), 2)
        if ok:
            self.assertTrue(tag.find(SELECTED)>=0)
        else:
            self.assertFalse(tag.find(SELECTED)>=0)

    def _check_sfmenu(self, response, occurence=1):
        self.assertEquals(str(response).count('class="sf-menu"'), occurence)
        
    def _create_menu(self, url, label, menu_type, draft=False, private=False, menu=None, target=None):
        if 'localeurl' in project_settings.INSTALLED_APPS:
            url = make_local(url)
            
        if not menu:
            menu = models.Menu()
            menu.label = '--'+label+'--'
            menu.div_id = "menubar"
            menu.save()
        
        menu_item = models.MenuItem()
        menu_item.label = '++'+label+'++'
        
        if menu_type == MENUITEM_URL:
            menu_item.url = url
            if target: menu_item.target = target
            
        elif menu_type == MENUITEM_CHILD:
            child_menu = models.Menu()
            child_menu.label = '##'+label+'##'
            child_menu.save()
    
            child_menu_item = models.MenuItem()
            child_menu_item.url = url
            child_menu_item.label = '..'+label+'..'
            child_menu_item.parent_menu = child_menu
            if target: child_menu_item.target = target
            child_menu_item.save()
            
            menu_item.child_menu = child_menu
            
        elif menu_type == MENUITEM_PAGE:
            dest_page = self._create_page(url, "Titre1234", "contenu1234")
            menu_item.page = dest_page
            if target: menu_item.target = target
            
        else:
            self.assertFalse(True) #raise an error

        menu_item.parent_menu = menu
        if draft: menu_item.published = False
        if private: menu_item.registration_required = True
        menu_item.save()
        return menu, label, url

    def _create_css_selector(self, selector):
        css_sel = models.CssSelector()
        css_sel.selector = selector
        css_sel.save()
        css_sel.site.add(project_settings.SITE_ID)
        css_sel.save()
        return css_sel

    def test_create_page(self):
        """Test a page can be created"""
        page = self._create_page("/", "Titre5487", "Contenu5555")
        self._check_page(page)

    def test_draft_page(self):
        """Test a page can be created as a draft"""
        page = self._create_page("/another-url", "Mon titre", "<h2>Titre2</h2><p>Contenu</p>")
        page.published = False
        page.save()
        
        self._check_page_error(page, 404, follow=False)
        
        self.client.login(username=STD_USER, password=PWD)
        self._check_page_error(page, 404, follow=False)
        
        self.client.login(username=STAFF_USER, password=PWD)
        self._check_page(page)

    def test_private_page(self):
        """Test a page can be created as a private"""
        page = self._create_page("/private/url/", "Confidentiel", "Secret attention")
        page.registration_required = True
        page.save()
    
        self._check_page_error(page, 302, follow=False) #redirect to login page
    
        self.client.login(username=STD_USER, password=PWD)
        self._check_page(page)
        
        self.client.login(username=STAFF_USER, password=PWD)
        self._check_page(page)
    
    def test_page_url(self):
        """Test a menu can be created"""
        regex = models.PageUrlFormField.validation_regex
        self.assert_(regex.match("/"))
        self.assert_(regex.match("/toto"))
        self.assert_(regex.match("/any/url/with/slashes"))
        self.assertFalse(regex.match("http://www.apidev.fr"))
        self.assertFalse(regex.match("http://apidev.fr"))
        self.assertFalse(regex.match("https://secret/"))
        self.assertFalse(regex.match("http://titi.fr?a=2%206&b=1"))
        self.assertFalse(regex.match("/+"))
        self.assertFalse(regex.match("/é"))
        self.assertFalse(regex.match("/a e"))
        self.assertFalse(regex.match("/c'est l'été"))
        self.assertFalse(regex.match("http://"))
        self.assertFalse(regex.match("toto"))
        self.assertFalse(regex.match("/test url"))
        self.assertFalse(regex.match("/?"))
        self.assertFalse(regex.match("/&"))
        self.assertFalse(regex.match("/%"))
        self.assertFalse(regex.match("/="))
    
    def test_add_menu(self):
        """Test a menu can be created"""
        menu = models.Menu()
        menu.label = 'Test'
        menu.save()
    
    def test_menu_ordering(self):
        """Test menu are sorted by ordering"""
        nums = [5, 1, 12, 2, 3, 8]
        for i in nums:
            menu = models.Menu()
            menu.label = 'Test'+str(i)
            menu.ordering = i
            menu.save()
        menus = models.Menu.objects.all()
        nums.sort()
        self.assertEqual([x.ordering for x in menus], nums)
    
    def test_menu_url(self):
        """Test a menu can be created"""
        regex = models.MenuUrlFormField.validation_regex
        self.assert_(regex.match("/"))
        self.assert_(regex.match("/toto"))
        self.assert_(regex.match("/any/url/with/slashes"))
        self.assert_(regex.match("http://www.apidev.fr"))
        self.assert_(regex.match("http://apidev.fr"))
        self.assert_(regex.match("https://secret/"))
        self.assert_(regex.match("http://titi.fr?a=2%206&b=1"))
        self.assertFalse(regex.match("/+"))
        self.assertFalse(regex.match("/é"))
        self.assertFalse(regex.match("/a e"))
        self.assertFalse(regex.match("/c'est l'été"))
        self.assertFalse(regex.match("http://"))
        self.assertFalse(regex.match("toto"))
        self.assertFalse(regex.match("http://test url"))
    
    def test_menu_on_page(self, menu_type=MENUITEM_URL):
        """Test a menu can be created"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type)
        page = self._create_page("/titi"+index, "Titi"+index, "Page"+index)
        page.config.menu.add(menu)
        page.save()
        response = self._check_page(page)
        self._check_menu(response, label, url)
    
    def test_child_menu_on_page(self):
        self.test_menu_on_page(MENUITEM_CHILD)
    
    def test_page_menu_on_page(self):
        self.test_menu_on_page(MENUITEM_PAGE)
    
    
    def test_menu_published(self, menu_type=MENUITEM_URL):
        """test the published flag on a menu"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type)
        menu.published = False
        menu.save()
        page = self._create_page("/titi"+index, "Titi"+index, "Page"+index)
        page.config.menu.add(menu)
        page.save()
    
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url, draft=True)
        
    def test_child_menu_published(self):
        self.test_menu_published(MENUITEM_CHILD)
    
    def test_page_menu_published(self):
        self.test_menu_published(MENUITEM_PAGE)
    
        
    def test_menu_is_private(self, menu_type=MENUITEM_URL):
        """test the registration_required flag on a menu"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type)
        menu.registration_required = True
        menu.save()
        page = self._create_page("/titi"+index, "Titi"+index, "Page"+index)
        page.config.menu.add(menu)
        page.save()
    
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url)
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url)
        
    def test_child_menu_is_private(self):
        self.test_menu_is_private(MENUITEM_CHILD)
    
    def test_page_menu_is_private(self):
        self.test_menu_is_private(MENUITEM_PAGE)
            
    def test_menu_is_private_and_draft(self, menu_type=MENUITEM_URL):
        """test the registration_required flag on a menu"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type)
        menu.registration_required = True
        menu.published = False
        menu.save()
        page = self._create_page("/titi"+index, "Titi"+index, "Page"+index)
        page.config.menu.add(menu)
        page.save()
    
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url, draft=True)
        
    def test_child_menu_is_private_and_draft(self):
        self.test_menu_is_private(MENUITEM_CHILD)
    
    def test_page_menu_is_private_and_draft(self):
        self.test_menu_is_private(MENUITEM_PAGE)
    
    def test_menuitem_is_draft(self, menu_type=MENUITEM_URL):
        """test the published flag on a menu item"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type, draft=True)
        menu, label2, url2 = self._create_menu("/tutu"+index, "Tutu"+index, menu_type, menu=menu)
        menu.save()
        page = self._create_page("/titi"+index, "Titi"+index, "Page"+index)
        page.config.menu.add(menu)
        page.save()
    
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
        self._check_menu(response, label2, url2)
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
        self._check_menu(response, label2, url2)
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url, draft=None)
        self._check_menu(response, label2, url2, draft=None)
        
        self._check_menu_item_is_draft(response, label, exists=True)
        self._check_menu_item_is_draft(response, label2, exists=False)
            
    def test_child_menuitem_is_draft(self):
        self.test_menuitem_is_draft(MENUITEM_CHILD)
    
    def test_page_menuitem_is_draft(self):
        self.test_menuitem_is_draft(MENUITEM_PAGE)
            
    def test_menuitem_is_private(self, menu_type=MENUITEM_URL):
        """test the registration_required flag on a menu item"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type, private=True)
        menu, label2, url2 = self._create_menu("/tutu"+index, "Tutu"+index, menu_type, menu=menu)
        menu.save()
        page = self._create_page("/titi"+index, "Titi"+index, "Page"+index)
        page.config.menu.add(menu)
        page.save()
    
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
        self._check_menu(response, label2, url2)
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url)
        self._check_menu(response, label2, url2)
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url)
        self._check_menu(response, label2, url2)
    
    def test_child_menuitem_is_private(self):
        self.test_menuitem_is_private(MENUITEM_CHILD)
    
    def test_page_menuitem_is_private(self):
        self.test_menuitem_is_private(MENUITEM_PAGE)
            
    def test_menuitem_is_private_and_draft(self, menu_type=MENUITEM_URL):
        """test the registration_required and published flag on a menu item"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type, private=True, draft=True)
        menu, label2, url2 = self._create_menu("/tutu"+index, "Tutu"+index, menu_type, menu=menu)
        menu.save()
        page = self._create_page("/titi"+index, "Titi"+index, "Page"+index)
        page.config.menu.add(menu)
        page.save()
    
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
        self._check_menu(response, label2, url2)
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url, visible=False)
        self._check_menu(response, label2, url2)
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_menu(response, label, url, draft=None)
        self._check_menu(response, label2, url2, draft=None)
        
        self._check_menu_item_is_draft(response, label, exists=True)
        self._check_menu_item_is_draft(response, label2, exists=False)
        
    def test_child_menuitem_is_private_and_draft(self):
        self.test_menuitem_is_private_and_draft(MENUITEM_CHILD)
    
    def test_page_menuitem_is_private_and_draft(self):
        self.test_menuitem_is_private_and_draft(MENUITEM_PAGE)
    
    def test_selected_menuitem(self, menu_type=MENUITEM_URL):
        """test the menu is seen as selected when the current page is displayed"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type)
        menu.save()
        if MENUITEM_PAGE != menu_type:
            page = self._create_page(url, "Une page", "Page"+index)
        else:
            page = models.CmsPage.objects.get(url=url)
    
        page2 = self._create_page("/titi", "Une autre page", "Page"+index)
        page2.save()
    
        #page and page2 have same page_config set by _create_page
        page.config.menu.add(menu)
        page.save()
    
        response = self._check_page(page)
        self._check_sfmenu(response, 1)
        self._check_menu_item_is_selected(response, label, menu_type, ok=True)
        
        response = self._check_page(page2)
        self._check_sfmenu(response, 1)
        self._check_menu_item_is_selected(response, label, menu_type, ok=False)
        
    def test_child_selected_menuitem(self):
        self.test_selected_menuitem(MENUITEM_CHILD)
    
    def test_page_selected_menuitem(self):
        self.test_selected_menuitem(MENUITEM_PAGE)
    
    def test_selected_draft_menuitem(self, menu_type=MENUITEM_URL):
        """test that draft menu is seen as selected when the current page is displayed"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type)
        menu.published = False
        menu.save()
        if MENUITEM_PAGE != menu_type:
            page = self._create_page(url, "Une page", "Page"+index)
        else:
            page = models.CmsPage.objects.get(url=url)
        page2 = self._create_page("/titi", "Une autre page", "Page"+index)
        page2.save()
    
        #page and page2 have same page_config set by _create_page
        page.config.menu.add(menu)
        page.save()
    
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_sfmenu(response, 1)
        self._check_menu_item_is_selected(response, label, menu_type, ok=True)
        
    def test_child_selected_draft_menuitem(self):
        self.test_selected_draft_menuitem(MENUITEM_CHILD)
    
    def test_page_selected_draft_menuitem(self):
        self.test_selected_draft_menuitem(MENUITEM_PAGE)
    
    def test_selected_private_menuitem(self, menu_type=MENUITEM_URL):
        """test that private menu is seen as selected when the current page is displayed"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type)
        menu.registration_required = True
        menu.save()
        if MENUITEM_PAGE != menu_type:
            page = self._create_page(url, "Une page", "Page"+index)
        else:
            page = models.CmsPage.objects.get(url=url)
    
        page2 = self._create_page("/titi", "Une autre page", "Page"+index)
        page2.save()
    
        #page and page2 have same page_config set by _create_page
        page.config.menu.add(menu)
        page.save()
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_sfmenu(response, 1)
        self._check_menu_item_is_selected(response, label, menu_type, ok=True)
        
    def test_child_selected_private_menuitem(self):
        self.test_selected_private_menuitem(MENUITEM_CHILD)
    
    def test_page_selected_private_menuitem(self):
        self.test_selected_private_menuitem(MENUITEM_PAGE)
    
    def test_menuitem_target(self, menu_type=MENUITEM_URL):
        """test the target on a menu item"""
        index = str(menu_type)
        menu, label, url = self._create_menu("/toto"+index, "Toto"+index, menu_type, target='target="_blank"')
        menu, label2, url2 = self._create_menu("/tutu"+index, "Tutu"+index, menu_type, menu=menu)
        menu.save()
        if MENUITEM_PAGE != menu_type:
            page = self._create_page(url, "Une page", "Page"+index)
        else:
            page = models.CmsPage.objects.get(url=url)
        
        page2 = self._create_page("/titi", "Une autre page", "Page"+index)
        page2.save()
        
        #page and page2 have same page_config set by _create_page
        page.config.menu.add(menu)
        page.save()
    
        response = self._check_page(page)
        self._check_sfmenu(response, 1)
        self._check_menu_item_target(response, label, menu_type, True)
        self._check_menu_item_target(response, label2, menu_type, False)
        
    def test_child_menuitem_target(self):
        self.test_menuitem_target(MENUITEM_CHILD)
    
    def test_page_menuitem_target(self):
        self.test_menuitem_target(MENUITEM_PAGE)
    
    def test_allmenus(self):
        """test a menu with the 3 different types"""
        menu, label, url = self._create_menu("/toto", "Toto", MENUITEM_URL)
        menu, label2, url2 = self._create_menu("/titi", "Titi", MENUITEM_PAGE, menu=menu)
        menu, label3, url3 = self._create_menu("/tutu", "Tutu", MENUITEM_CHILD, menu=menu)
        menu.save()
        
        page = self._create_page(url, "Une page", "Avec quelquechose")
        page.config.menu.add(menu)
        page.save()
        
        response = self._check_page(page)
        self._check_sfmenu(response, 1)
        self._check_menu(response, label, url)
        self._check_menu(response, label2, url2)
        self._check_menu(response, label3, url3)
        
    def test_menu_with_50_items(self):
        """test a menu with the 50 different items"""
        menu, label, url = self._create_menu("/toto0", "Toto0", MENUITEM_URL)
        labels_and_urls = [(label, url)]
        for i in range(1, 50):
            menu, label, url = self._create_menu("/toto{0}".format(i), "Toto{0}".format(i), MENUITEM_PAGE, menu=menu)
        menu.save()
        
        page = self._create_page("/toto0", "Une page", "Avec quelquechose")
        page.config.menu.add(menu)
        page.save()
        
        response = self._check_page(page)
        self._check_sfmenu(response, 1)
        for i in range(50):
            self._check_menu(response, "Toto{0}".format(i), "/toto{0}".format(i))
        
    def test_menu_not_in_div(self):
        """test a menu with the 50 different items"""
        menu, label, url = self._create_menu("/toto", "Toto", MENUITEM_URL)
        menu.div_id = "blabla"
        menu.save()
        
        page = self._create_page(url, "Une page", "Avec quelquechose")
        page.config.menu.add(menu)
        page.save()
        
        response = self._check_page(page)
        self._check_sfmenu(response, 0)
        self._check_menu(response, label, url, visible=False)
        
    def _create_plugin(self, name, content):
        plugin = models.Plugin()
        plugin.name = name
        plugin.content = content
        plugin.save()
        return plugin
    
    def _check_plugin(self, response, plugin, visible=True, draft=False):
        if visible:
            self.assertContains(response, plugin.content)
            self._check_tag(response, plugin.content, DRAFT, draft)
        else:
            self.assertNotContains(response, plugin.content)
        
    def test_plugin(self):
        """test plugin is dsiplayed"""
        plugin = self._create_plugin("toto", "hello guys")
        
        page = self._create_page("/toto", "Une page", "Avec quelquechose")
        page.config.plugins.add(plugin)
        page.save()
        
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=True)
        
    def test_draft_plugin(self):
        """test draft plugin"""
        plugin = self._create_plugin("toto", "hello guys")
        plugin.published = False
        plugin.save()
        
        page = self._create_page("/toto", "Une page", "Avec quelquechose")
        page.config.plugins.add(plugin)
        page.save()
        
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=False)
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=False)
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=True, draft=True)
    
    def test_divid_plugin(self):
        """test div_id of a plugin"""
        plugin = self._create_plugin("toto", "hello guys")
        plugin.div_id = "blabla"
        plugin.save()
        
        page = self._create_page("/toto", "Une page", "Avec quelquechose")
        page.config.plugins.add(plugin)
        page.save()
        
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=False)
        
    def test_private_plugin(self):
        """test private plugin"""
        plugin = self._create_plugin("toto", "hello guys")
        plugin.registration_required = True
        plugin.save()
        
        page = self._create_page("/toto", "Une page", "Avec quelquechose")
        page.config.plugins.add(plugin)
        page.save()
        
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=False)
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=True, draft=False)
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=True, draft=False)
        
    def test_private_plugin(self):
        """test private plugin"""
        plugin = self._create_plugin("toto", "hello guys")
        plugin.registration_required = True
        plugin.save()
        
        page = self._create_page("/toto", "Une page", "Avec quelquechose")
        page.config.plugins.add(plugin)
        page.save()
        
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=False)
    
        self.client.login(username=STD_USER, password=PWD)
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=True, draft=False)
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        self._check_plugin(response, plugin, visible=True, draft=False)
    
    def test_plugin_ordering(self):
        """test private plugin"""
        nums = [5, 1, 3, 2, 4]
        plugins = []
        for i in nums:
            plugin = self._create_plugin("toto{0}".format(i), "--{0}--".format(i))
            plugin.ordering = i
            plugin.save()
            plugins.append(plugin)
        
        page = self._create_page("/toto", "Une page", "Avec quelquechose")
        for p in plugins: page.config.plugins.add(p)
        page.save()
        
        response = self._check_page(page)
        for p in plugins:
            self._check_plugin(response, p)
            
        content = str(response)
        plugins_content = [p.content for p in plugins]
        plugins_content.sort()
        plugins_order = [content.find(x) for x in plugins_content]
        
        self.assertEqual(len(plugins_order), len(nums))
        plugins_order_sorted = plugins_order[:]
        plugins_order_sorted.sort()
        self.assertEqual(plugins_order_sorted, plugins_order)
        
    def test_metaheaders(self):
        """Check that meta headers are displayed properly"""
        
        site_config = self._create_site_config('mysite')
        
        page = self._create_page("/", "Titre5487", "Contenu5555")
        page.save()
        
        meta1 = models.MetaHeader()
        meta1.site_config = site_config
        meta1.name = "my-meta-meta"
        meta1.content = "ayytyugb,b5458212%%opopn!!%"
        meta1.save()
        
        meta2 = models.MetaHeader()
        meta2.site_config = site_config
        meta2.name = "my-meta-meta"
        meta2.content = "ayytyugb,b5458212%%opopn!!%"
        meta2.save()
        
        response = self._check_page(page)
        self.assertContains(response, meta1)
        self.assertContains(response, meta2)
    
    def test_css(self):
        """Check that css page is create properly"""
        page = self._create_page("/", "Titre5487", "Contenu5555")
            
        css_sel = self._create_css_selector("#my-id")
        
        css_prop = models.CssProperty()
        css_prop.selector = css_sel
        css_prop.property = "background"
        css_prop.value = "red"
        css_prop.save()
        
        css_url = reverse('flatcms.views.get_css_style')
        response = self.client.get(css_url)
        
        self.assertContains(response, "#my-id {")
        self.assertContains(response, "background: red;")
        self.assertContains(response, "}")
        
        response = self._check_page(page)
        self.assertContains(response, css_url)
    
    def test_disabled_css(self):
        """Check that css page is create properly"""
        page = self._create_page("/", "Titre5487", "Contenu5555")
        
        css_sel = self._create_css_selector("#my-id")
        css_sel.enabled = False
        css_sel.save()
        
        css_prop = models.CssProperty()
        css_prop.selector = css_sel
        css_prop.property = "background"
        css_prop.value = "red"
        css_prop.save()
        
        css_url = reverse('flatcms.views.get_css_style')
        response = self.client.get(css_url)
        
        self.assertNotContains(response, "#my-id")
        self.assertNotContains(response, "background: red;")
    
    def test_css_in_editor(self):
        """Check that css page is created properly"""
        page = self._create_page("/", "Titre5487", "Contenu5555")
        
        sel_in = self._create_css_selector("#in_editor")
        sel_in.in_editor = True
        sel_in.save()
        
        css_prop = models.CssProperty()
        css_prop.selector = sel_in
        css_prop.property = "background"
        css_prop.value = "red"
        css_prop.save()
                
        sel_out = self._create_css_selector("#out_editor")
        #sel_out.in_editor = False
        sel_out.save()
        
        css_prop = models.CssProperty()
        css_prop.selector = sel_out
        css_prop.property = "color"
        css_prop.value = "green"
        css_prop.save()
        
        css_url = reverse('flatcms.views.get_css_style')
        response = self.client.get(css_url)
        self.assertContains(response, "#in_editor")
        self.assertContains(response, "#out_editor")
        self.assertContains(response, "background: red;")
        self.assertContains(response, "color: green;")
        
        css_url = reverse('flatcms.views.get_css_style_for_mce')
        response = self.client.get(css_url)
        self.assertContains(response, "#in_editor")
        self.assertNotContains(response, "#out_editor")
        self.assertContains(response, "background: red;")
        self.assertNotContains(response, "color: green;")
        
    
    def test_ordering_css(self):
        """Check that css page is created properly"""
        page = self._create_page("/", "Titre5487", "Contenu5555")
        
        nums = [7, 2, 5, 1, 3]
        css_sels = []
        for i in nums:
            css_sel = self._create_css_selector("#"+str(i))
            css_sel.ordering = i
            css_sel.save()
            
            css_sels.append(css_sel)
        
            css_prop = models.CssProperty()
            css_prop.selector = css_sel
            css_prop.property = "background"
            css_prop.value = "red"
            css_prop.save()
        
        css_url = reverse('flatcms.views.get_css_style')
        response = self.client.get(css_url)
        pos = []
        nums.sort()
        for i in nums:
            self.assertContains(response, "#"+str(i))
            pos.append(response.content.find("#"+str(i)))
        
        pos_sorted = pos[:]
        pos_sorted.sort()
        
        self.assertEqual(pos, pos_sorted)

    def test_alias(self):
        """Check that an alias returns the right page"""
        page = self._create_page("/", "Titre5487", "Contenu5555")
    
        page2 = self._create_page("/titi", "Titi", "Tiritiit")
        page2.save()
        
        alias = models.Alias()
        alias.url = make_local("/toto")
        alias.page = page
        alias.save()
        
        response1 = self.client.get(alias.url, follow=True)
        response2 = self._check_page(page)
        
        def _get_response_content(response):
            content = str(response)
            pos1 = content.find("<body>")
            pos2 = content.find("</body>", pos1)
            return content[pos1:pos2]
        
        self.assertEqual(_get_response_content(response1), _get_response_content(response2))
        
    def test_sitemap(self):
        """Check the sitemap"""
        dt = datetime.datetime.now().strftime("%Y-%m-%d")
        page0 = self._create_page("/", "Titre5487", "Contenu5555")
        page0.changefreq = "daily"
        page0.lastmod = dt
        page0.priority = "0.2"
        page0.save()
        
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        page1.changefreq = "weekly"
        page1.lastmod = dt
        page1.priority = "0.8"
        page1.save()
        
        page2 = self._create_page("/page/2", "Page2", "La page no 2")
        page2.save()
        
        page3 = self._create_page("/toto", "Toto", "Salut toto")
        page3.save()
        
        page4 = self._create_page("/azerty", "Azerty", "french keyboard")
        page4.sitemap = False
        page4.save()
        
        site = Site.objects.get_current()
        alias1 = models.Alias.objects.create(site=site, url = make_local("/titi/1"), page = page3, sitemap=True)
        alias2 = models.Alias.objects.create(site=site, url = make_local("/titi/2"), page = page3, sitemap=False)
        alias3 = models.Alias.objects.create(site=site, url = make_local("/titi/3"), page = page4, sitemap=True)
        alias4 = models.Alias.objects.create(site=site, url = make_local("/titi/4"), page = page4, sitemap=False)
        
        sitemap_url = reverse('sitemap')
        response = self.client.get(sitemap_url)
        
        for page in (page0, page1, page2, page3):
            self.assertContains(response, page.url)
            
        self.assertContains(response, page0.priority)
        self.assertContains(response, dt)
        self.assertContains(response, page0.changefreq)
                
        self.assertNotContains(response, page4.url)
            
        self.assertContains(response, alias1.url)
        self.assertContains(response, alias3.url)
        self.assertNotContains(response, alias2.url)
        self.assertNotContains(response, alias4.url)
    
    def test_sitemap_multisite(self):
        """Check that pages are only in the sitemap of their site"""
        site2 = Site()
        site2.domain = 'toto.fr'
        site2.name = 'toto'
        site2.save()
        
        site_config1 = self._create_site_config('mysite')
        site_config2 = self._create_site_config('myothersite', site2)
        
        dt = datetime.datetime.now().strftime("%Y-%m-%d")
        page0 = self._create_page("/", "Titre5487", "Contenu5555")
        page0.changefreq = "daily"
        page0.lastmod = dt
        page0.priority = "0.2"
        page0.save()
        
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        page1.changefreq = "weekly"
        page1.lastmod = dt
        page1.priority = "0.8"
        page1.site = site2
        page1.save()
                
        sitemap_url = reverse('sitemap')
        
        response = self.client.get(sitemap_url)
        
        self.assertContains(response, page0.url)
        
        self.assertNotContains(response, page1.url)
            
    def test_sitemap_alias_multisite(self):
        """Check that alias are only in the sitemap of their site"""
        site2 = Site()
        site2.domain = 'toto.fr'
        site2.name = 'toto'
        site2.save()
        
        site_config1 = self._create_site_config('mysite')
        site_config2 = self._create_site_config('myothersite', site2)
        
        dt = datetime.datetime.now().strftime("%Y-%m-%d")
        page0 = self._create_page("/", "Titre5487", "Contenu5555")
        page0.changefreq = "daily"
        page0.lastmod = dt
        page0.priority = "0.2"
        page0.save()
        
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        page1.changefreq = "weekly"
        page1.lastmod = dt
        page1.priority = "0.8"
        page1.site = site2
        page1.save()
        
        alias0 = models.Alias()
        alias0.url = make_local("/titi")
        alias0.page = page1
        alias0.save()
        
        alias1 = models.Alias()
        alias1.url = make_local("/toto")
        alias1.site = site2
        alias1.page = page0
        alias1.save()
        
        sitemap_url = reverse('sitemap')
        response = self.client.get(sitemap_url)
        
        self.assertContains(response, alias0.url)
        
        self.assertNotContains(response, alias1.url)
    
    def test_multisite(self):
        """Check that page are retrieved according to the sitemap"""
        site1 = Site.objects.get(id=project_settings.SITE_ID)
        
        site2 = Site()
        site2.name = "test"
        site2.domain = "test.fr"
        site2.save()
        
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        page1.site = site1
        page1.save()
        
        page2 = self._create_page("/page/2", "Page2", "La page no 2")
        page2.site = site2
        page2.save()
        
        page3 = self._create_page("/toto", "Toto", "Salut toto")
        page3.site = site1
        page3.save()
        
        for page in (page1, page3):
            self._check_page(page)
        
        response = self.client.get(page2.url)
        self.assertEqual(response.status_code, 404)
        
    def test_alias_multisite(self):
        """Check that an alias returns the right page"""
        site1 = Site.objects.get_current()
        
        site2 = Site()
        site2.name = "test"
        site2.domain = "test.fr"
        site2.save()
        
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        page1.site = site1
        page1.save()
        
        page2 = self._create_page("/page/2", "Page2", "La page no 2")
        page2.site = site2
        page2.save()
        
        alias1 = models.Alias()
        alias1.url = make_local("/alias/1")
        alias1.site = site1
        alias1.page = page1
        alias1.save()
        
        alias2 = models.Alias()
        alias2.url = make_local("/alias/2")
        alias2.site = site1
        alias2.page = page2
        alias2.save()
        
        alias3 = models.Alias()
        alias3.url = make_local("/alias/3")
        alias3.site = site2
        alias3.page = page2
        alias3.save()
                
        response1 = self.client.get(alias1.url, follow=True)
        response2 = self.client.get(alias2.url, follow=True)
        response3 = self.client.get(alias3.url, follow=True)
        
        self.assertEqual(200, response1.status_code)        
        self.assertContains(response1, site1.domain)
        
        self.assertEqual(200, response2.status_code)        
        self.assertContains(response2, site1.domain)
        
        self.assertEqual(404, response3.status_code)
    
    def test_alias_redirect(self):
        """Check that the page is redirected if the flag is on"""
        site1 = Site.objects.get_current()
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        page1.site = site1
        page1.save()
        
        alias1 = models.Alias()
        alias1.url = make_local("/alias/1")
        alias1.site = site1
        alias1.page = page1
        alias1.redirect = True
        alias1.save()
        
        response1 = self.client.get(alias1.url)
        self.assertEqual(302, response1.status_code)

        url = u'http://{0}{1}'.format(site1.domain, page1.url)
        response1 = self.client.get(alias1.url, follow=True)
        self.assertRedirects(response1, url)
        
    def test_alias_config(self):
        """Check that the page is redirected if the flag is on"""
        site1 = Site.objects.get_current()
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        page1.site = site1
        page1.save()
        
        page_plugin = self._create_plugin("PagePlugin", "Plugin de la page")
        page1.config.plugins.add(page_plugin)
        page1.config.save()
        
        alias_config = models.PageConfig()
        alias_config.name = "AliasPlugin"
        alias_config.save()
        
        alias_plugin = self._create_plugin("AliasPlugin", "Plugin de l'alias")
        alias_config.plugins.add(alias_plugin)
        alias_config.save()
        
        alias1 = models.Alias()
        alias1.url = make_local("/alias/1")
        alias1.site = site1
        alias1.page = page1
        alias1.config = alias_config
        alias1.save()

        response1 = self.client.get(alias1.url)
        self.assertContains(response1, alias_plugin.content)
        self.assertNotContains(response1, page_plugin.content)

   
    def test_duplicated_page(self):
        """Check that an error is raised when 2 pages of the same site have the same URL"""
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        
        page2 = self._create_page("/page/2", "Page2", "La page no 2")
        page2.url = page1.url
        
        self.assertRaises(IntegrityError, page2.save)
        
    def test_duplicated_alias(self):
        """Check that an error is raised when 2 alias of the same site have the same URL"""
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        
        alias1 = models.Alias()
        alias1.url = make_local("/alias/1")
        alias1.site = page1.site
        alias1.page = page1
        alias1.save()
        
        alias2 = models.Alias()
        alias2.url = make_local("/alias/1")
        alias2.site = page1.site
        alias2.page = page1
          
        self.assertRaises(IntegrityError, alias2.save)
        
    def test_page_not_found_by_site(self):
        """Check that if the site is not the currrent one the page is not found"""
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        
        site2 = Site()
        site2.name = "test"
        site2.domain = "test.fr"
        site2.save()
        
        page1.site = site2
        page1.save()
         
        response = self.client.get(page1.url)
        self.assertEqual(404, response.status_code)
    
    def test_alias_not_found_by_site(self):
        """Check that if the site is not the currrent one the alias is not found"""
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        
        site2 = Site()
        site2.name = "test"
        site2.domain = "test.fr"
        site2.save()
        
        alias1 = models.Alias()
        alias1.url = make_local("/alias/1")
        alias1.site = site2
        alias1.page = page1
        alias1.save()
        
        response = self.client.get(alias1.url, follow=True)
        self.assertEqual(404, response.status_code)
        
    def test_alias_intersite(self):
        """Check that an alias can return a page of a different site"""
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        
        site2 = Site()
        site2.name = "test"
        site2.domain = "test.fr"
        site2.save()
        
        alias1 = models.Alias()
        alias1.url = make_local("/alias/1")
        alias1.site = page1.site
        alias1.page = page1
        alias1.save()
        
        page1.site = site2
        page1.save()
        
        response = self.client.get(alias1.url, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, 'Page1')

    def test_page_prio_on_alias(self):
        """Check that an error is raised when 2 pages of the same site have the same URL"""
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        
        page2 = self._create_page("/page/2", "Page2", "La page no 2")
        
        alias1 = models.Alias()
        alias1.url = page1.url
        alias1.page = page2
        alias1.save()
        
        response = self.client.get(alias1.url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, 'Page1')
        self.assertNotContains(response, 'Page2')
            
    def test_page_template(self):
        """Check that the right template is used in different situations"""
        
        default_template = 'flatcms.unittest' #threecols
        site_config = models.SiteConfig()
        site_config.name = "TestConfig"
        config = models.PageConfig()
        config.name = "Default"
        config.lang = 'en'
        config.template_name = "flatcms.twocols"
        config.save()
        site_config.site = Site.objects.get_current()
        site_config.default_config = config
        site_config.save()
        
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        page1.config.template_name = "flatcms.onecol"
        page1.config.save()
        
        response = self._check_page(page1)
        self.assertTemplateUsed(response, views.get_template_name(page1.config.template_name))
        #self.assertTemplateNotUsed(response, views.get_template_name(site_config.default_config.template_name))
        
        page1.config = None
        page1.save()
        
        response = self._check_page(page1)
        self.assertEquals(200, response.status_code)
        self.assertTemplateUsed(response, views.get_template_name(site_config.default_config.template_name))
        
        site_config.default_config.template_name = ""
        site_config.default_config.save()
        response = self._check_page(page1)
        self.assertTemplateUsed(response, views.get_template_name(default_template))
        
    def test_duplicate_site_config(self):
        """Check that a site has only one config"""
        
        site_config1 = models.SiteConfig()
        site_config1.name = "Config1"
        site_config1.site = Site.objects.get(id=project_settings.SITE_ID)
        config = models.PageConfig()
        config.name = "Default"
        config.template_name = "flatcms.twocols"
        config.save()
        site_config1.default_config = config
        site_config1.save()
        
        site_config2 = models.SiteConfig()
        site_config2.name = "Config2"
        site_config2.site = Site.objects.get(id=project_settings.SITE_ID)
        config2 = models.PageConfig()
        config2.name = "Other"
        config2.template_name = "flatcms.threecols"
        config2.save()
        site_config2.default_config = config
        self.assertRaises(Exception, site_config2.save)
                
    def test_site_copyright(self):
        """Check that the copyright block includes optional designer information"""
        site_config1 = models.SiteConfig()
        site_config1.name = "Config1"
        config = models.PageConfig()
        config.name = "Default"
        config.template_name = "flatcms.twocols"
        config.save()
        site_config1.default_config = config
        site_config1.save()
        
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        page1.config.template_name = ""
        page1.config.save()
        
        response = self._check_page(page1)
        self.assertNotContains(response, COPYRIGHT_MARK)
        
        site_config1.designer = "Apidev"
        site_config1.designer_website = "http://www.apidev.fr"
        site_config1.save()
        self.assertEqual(site_config1.design_year, datetime.date.today().strftime("%Y"))
        site_config1.design_year = "1975"
        site_config1.save()
        
        response = self._check_page(page1)
        self.assertContains(response, COPYRIGHT_MARK)
        self.assertContains(response, site_config1.designer)
        self.assertContains(response, site_config1.designer_website)
        self.assertContains(response, site_config1.design_year)
        
    def _get_steps(self, response):
        soup = BeautifulSoup(response.content)
        result = []
        tag = soup.find(id='stepbanner') 
        
        for step in tag.findAll('span'):
            classes = []
            url = None
            if step.a:
                val = step.a.contents[0]
                num = None
                for (attr, value) in step.a.attrs:
                    if attr=='href':
                        url = value
                if step.span:
                    num = int(step.span.contents[0][:-1])
            elif not step.span:
                val = step.contents[0]
                num = None
            else:
                num = int(step.span.contents[0][:-1])
                val = step.contents[1]
            for (attr, value) in step.attrs:
                if attr=='class':
                    classes = value.strip().split(' ')
            result.append((val, classes, num, url))
        steps = []
        for (v, cl, num, url) in result:
            if set(cl).intersection(['step']):
                try:
                    cl.index('selitem')
                    sel = True
                except:
                    sel = False
                steps.append({'text':v, 'sel':sel, 'num':num, 'url':url})
        return steps
        
    def _create_steps_page(self, steps, show_all=False, show_num=False, level=None):
        page1 = self._create_page("/page/1", "Page1", "La page no 1")
        if None==level:
            level=len(steps)
        b = models.StepBanner.objects.create(name='etapes', show_num=show_num,
                                             show_all=show_all)
        
        for i, l in enumerate(steps):
            s = models.Step()
            if type(l) is tuple:
                s.label = l[0]
                if type(l[1]) is str:
                    s.url = l[1]
                else:
                    s.page = l[1]
                if len(l)>2:
                    s.page = l[2]
            else:    
                s.label = l
            s.level = i+1
            s.banner = b
            s.save()
        
        page1.step_level = level
        page1.save()
        
        page1.config.step_banner = b
        page1.config.save()
        return page1
        
    def test_steps(self):
        """check that the steps are displayed when attached to a page"""
        steps = ['A', 'B', 'C']
        page1 = self._create_steps_page(steps)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], steps)
        self.assertEqual([v['num'] for v in result], [None for i in steps])
        self.assertEqual([v['sel'] for v in result],
            [page1.step_level==i+1 for i in range(len(steps))])
            
    def test_step_with_num(self):
        """check that the steps are displayed with num"""
        steps = ['A', 'B', 'C']
        page1 = self._create_steps_page(steps, show_num=True)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], steps)
        self.assertEqual([v['num'] for v in result], range(1, len(steps)+1))
        self.assertEqual([v['sel'] for v in result],
            [page1.step_level==i+1 for i in range(len(steps))])
    
    def test_step_current(self):
        """check that the steps level is working ok"""
        steps = ['A', 'B', 'C']
        page1 = self._create_steps_page(steps, show_num=True, level=2)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], steps[:page1.step_level])
        self.assertEqual([v['num'] for v in result], range(1, page1.step_level+1))
        self.assertEqual([v['sel'] for v in result],
            [page1.step_level==i+1 for i in range(page1.step_level)])
            
    def test_step_current_showall(self):
        """check that the steps level in show_all mode"""
        steps = ['A', 'B', 'C']
        page1 = self._create_steps_page(steps, show_all=True, level=2)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], steps)
        self.assertEqual([v['sel'] for v in result],
            [page1.step_level==i+1 for i in range(len(steps))])
    
    def test_step_link(self):
        """check that the links of a step"""
        steps = [('A', '/A'), ('B', '/B'), 'C']
        page1 = self._create_steps_page(steps, level=2)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], ['A', 'B'])
        self.assertEqual([v['sel'] for v in result], [False, True])
        self.assertEqual([v['url'] for v in result], ['/A', None])
        
    def test_step_link_showall(self):
        """check that the links of a step in show_all mode"""
        steps = [('A', '/A'), ('B', '/B'), 'C']
        page1 = self._create_steps_page(steps, show_all=True, level=2)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], ['A', 'B', 'C'])
        self.assertEqual([v['sel'] for v in result], [False, True, False])
        self.assertEqual([v['url'] for v in result], ['/A', None, None])
    
    def test_step_page(self):
        """check that the page of a step"""
        page2 = self._create_page('/page/2', 'title', 'content')
        page3 = self._create_page('/page/3', 'title', 'content')
        steps = [('A', page2), ('B', page3), 'C']
        page1 = self._create_steps_page(steps, level=2)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], ['A', 'B'])
        self.assertEqual([v['sel'] for v in result], [False, True])
        self.assertEqual([v['url'] for v in result], [page2.url, None])
    
    def test_step_page_showall(self):
        """check that the page of a step showall mode"""
        page2 = self._create_page('/page/2', 'title', 'content')
        page3 = self._create_page('/page/3', 'title', 'content')
        steps = [('A', page2), ('B', page3), 'C']
        page1 = self._create_steps_page(steps, show_all=True, level=2)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], ['A', 'B', 'C'])
        self.assertEqual([v['sel'] for v in result], [False, True, False])
        self.assertEqual([v['url'] for v in result], [page2.url, None, None])
        
    def test_step_link_prio_on_page(self):
        """check behavior when links  and page are defined on a step"""
        page2 = self._create_page('/page/2', 'title', 'content')
        page3 = self._create_page('/page/3', 'title', 'content')
        steps = [('A', '/url/1', page2), ('B', '/url/2', page3), ('C', 'url/3')]
        page1 = self._create_steps_page(steps)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], ['A', 'B', 'C'])
        self.assertEqual([v['sel'] for v in result], [False, False, True])
        self.assertEqual([v['url'] for v in result], ['/url/1', '/url/2', None])
        
    def test_step_link_and_num(self):
        """check behavior when links and num"""
        steps = [('A', '/url/1'), ('B', '/url/2'), ('C', 'url/3')]
        page1 = self._create_steps_page(steps, show_num=True)
        response = self._check_page(page1)
        result = self._get_steps(response)    
        self.assertEqual([v['text'] for v in result], ['A', 'B', 'C'])
        self.assertEqual([v['sel'] for v in result], [False, False, True])
        self.assertEqual([v['num'] for v in result], [1, 2, 3])
        self.assertEqual([v['url'] for v in result], ['/url/1', '/url/2', None])
    
    def test_app_only(self):
        """check the app_only flag on CmsPage"""
        page1 = self._create_page('/url/1', 'title1', 'content1')
        page1.app_only = True
        page1.save()
        
        response = self.client.get(page1.url)
        self.assertEqual(404, response.status_code)
        
        page1.app_only = False
        page1.save()
        
        response = self.client.get(page1.url)
        self.assertEqual(200, response.status_code)
        
    def test_404(self):
        """check the 404 page"""
        
        page = self._create_page(make_local('/'), 'hello', 'world')
        
        page_config = models.PageConfig()
        page_config.name = '404'
        page_config.save()
        
        site_config = models.SiteConfig.objects.get(site__id=project_settings.SITE_ID)
        site_config.default_config = page_config
        site_config.save()
            
        plugin = models.Plugin()
        plugin.div_id = 'footer'
        plugin.name = 'toto'
        plugin.content = 'Rien de bien important'
        plugin.save()
        
        page_config.plugins.add(plugin)
        page_config.save()
        
        response = self.client.get(make_local('/ghost-page'))
        self.assertEqual(404, response.status_code)
        self.assertTemplateUsed(response, '404.html')
        self.assertTrue(response.content.find(plugin.content)>0)
        self.assertFalse(response.content.find(page.content)>0)
        
    def test_site_config(self):
        """check that the site_config is retrieved properly"""
        site1 = Site.objects.get_current()
        site1.domain = 'toto.fr'
        site1.name = 'toto'
        site1.save()
        
        site2 = Site()
        site2.domain = 'titi.fr'
        site2.name = 'titi'
        site2.save()
                
        site_config1 = models.SiteConfig()
        config = models.PageConfig()
        config.name = "Default"
        config.save()
        site_config1.default_config = config

        site_config1.name = "Config Titi"
        site_config1.site = site2
        site_config1.save()
    
        site_config2 = models.SiteConfig()
        site_config2.name = "Config Toto"
        site_config2.site = site1
        site_config2.default_config = config
        site_config2.save()
        
        page1 = self._create_page('/', 'azerty', 'AZERTY')
        
        response = self._check_page(page1)
        self.assertContains(response, 'toto.fr')
        self.assertNotContains(response, 'titi.fr')
        
    def test_without_google_analytics(self):
        """check that there is no reference to Google Analytics if no code is given"""
        page1 = self._create_page('/', 'azerty', 'AZERTY')
        site_config = models.SiteConfig.objects.get(site=Site.objects.get_current())
        site_config.analytics_code = ''
        site_config.analytics_option = 'disabled'
        site_config.save()
        response = self._check_page(page1)
        self.assertContains(response, 'AZERTY')
        self.assertNotContains(response, 'oogle')
        
    def test_no_google_analytics_if_not_published(self):
        """check that there is no reference to Google Analytics if the page is not published"""
        page1 = self._create_page('/', 'azerty', 'AZERTY')
        site_config = models.SiteConfig.objects.get(site=Site.objects.get_current())
        site_config.analytics_code = 'ABCDEFGH'
        site_config.analytics_option = 'single'
        site_config.save()
        page1.published = False
        page1.save()
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page1)
        self.assertContains(response, 'AZERTY')
        self.assertContains(response, _("Google Analytics disabled"))
        
    def test_google_analytics(self):
        """check that the Google Analytics code is included when defined"""
        page1 = self._create_page('/', 'azerty', 'AZERTY')
        options = ('single', 'single-and-sub', 'multi')
        keywords = ('', "_setDomainName', '."+Site.objects.get_current().domain, '_setAllowLinker')
        for option, keyword in zip(options, keywords):
            site_config = models.SiteConfig.objects.get(site=Site.objects.get_current())
            site_config.analytics_code = 'ABCDEFGH'
            site_config.analytics_option = option
            site_config.save()
            response = self._check_page(page1)
            self.assertContains(response, 'AZERTY')
            self.assertContains(response, 'ABCDEFGH')
            self.assertContains(response, '.google-analytics.com/ga.js')
            if keyword:
                self.assertContains(response, keyword)
            for forbidden in (set(keywords)-set([keyword])):
                if forbidden:
                    self.assertNotContains(response, forbidden)
        
    def _make_links_val(self, url, title, published):
        flag = u'!-' if not published else ''
        return '"{2} [{0}{1}]", "{2}"'.format(flag, title, url)

    def test_link_list(self):
        #check that a js page is generated with CmsPage Links
        page1 = self._create_page('/toto', 'Toto', 'ok')
        page2 = self._create_page('/titi', 'Titi', 'ok')
        response = self.client.get(reverse('linklist.js',
            kwargs={'app_name':'flatcms'}), follow=True)
        self.assertContains(response, self._make_links_val(page1.url, page1.title, page1.published))
        self.assertContains(response, self._make_links_val(page2.url, page2.title, page2.published))
    
    def test_link_list_invalid_chars_in_title(self):
        #check that a js page is generated with CmsPage Links
        #Check ' and " in title
        page1 = self._create_page('/toto', '"Salut"', 'ok')
        page2 = self._create_page('/titi', "C'est l'été", 'ok')
        response = self.client.get(reverse('linklist.js',
            kwargs={'app_name':'flatcms'}), follow=True)
        self.assertContains(response, self._make_links_val(page1.url,
            page1.title.replace('"', '\\"'), page1.published))
        self.assertContains(response, self._make_links_val(page2.url,
            page2.title.replace('"', '\\"'), page2.published))
        
    def test_link_list_alias(self):
        #check that a js page is generated with Alias Links
        page = self._create_page('/toto', '"Salut"', 'ok')
        alias = models.Alias()
        alias.url = "/titi"
        alias.page = page
        alias.save()
        response = self.client.get(reverse('linklist.js',
            kwargs={'app_name':'flatcms'}), follow=True)
        self.assertContains(response, self._make_links_val(page.url,
            page.title.replace('"', '\\"'), page.published))
        self.assertContains(response, self._make_links_val(alias.url,
            page.title.replace('"', '\\"'), page.published))
        
    def test_link_list_unpublished(self):
        #check that a js page is generated with Alias Links
        page = self._create_page('/toto', '"Salut"', 'ok')
        page.published = False
        page.save()
        alias = models.Alias()
        alias.url = "/titi"
        alias.page = page
        alias.save()
        response = self.client.get(reverse('linklist.js',
            kwargs={'app_name':'flatcms'}), follow=True)
        self.assertContains(response, self._make_links_val(page.url,
            page.title.replace('"', '\\"'), page.published))
        self.assertContains(response, self._make_links_val(alias.url,
            page.title.replace('"', '\\"'), page.published))
        
    def test_tiny_mce_config(self):
        #Check that Tiny MCE config file can be retrieved
        self._create_site_config('mysite')
        url = '/flatcms/js/tinymce_config.js'
        self.assertEqual(url, reverse('tinymce_config.js'))
        self.assertEqual(200, self.client.get(url).status_code)
        
    def test_breadcrumbs(self):
        page1 = self._create_page('/toto', 'toto', 'toto')
        page1.breadcrumb_text = 'Top level'
        page1.save()
        
        page2 = self._create_page('/titi', 'toto', 'toto')
        page2.breadcrumb_text = 'Upper level'
        page2.breadcrumb_parent = page1
        page2.save()

        page3 = self._create_page('/tutu', 'toto', 'toto')
        page3.breadcrumb_text = 'Current level'
        page3.breadcrumb_parent = page2
        page3.save()
        
        response = self.client.get(page3.url)
        self.assertContains(response, page1.breadcrumb_text)
        self.assertContains(response, page1.url)
        self.assertContains(response, page2.breadcrumb_text)
        self.assertContains(response, page2.url)
        self.assertContains(response, page3.breadcrumb_text)
        
        response = self.client.get(page2.url)
        self.assertContains(response, page1.breadcrumb_text)
        self.assertContains(response, page1.url)
        self.assertContains(response, page2.breadcrumb_text)
        self.assertNotContains(response, page3.breadcrumb_text)
        
    def test_breadcrumbs_no_dead_loops(self):
        page1 = self._create_page('/toto', 'toto', 'toto')
        page1.breadcrumb_text = 'Top level'
        page1.save()
        page1.breadcrumb_parent = page1
        page1.save()
        
        response = self.client.get(page1.url)
        self.assertEquals(response.status_code, 200)
        
        page2 = self._create_page('/titi', 'toto', 'toto')
        page2.breadcrumb_text = 'Upper level'
        page2.breadcrumb_parent = page1
        page2.save()
        
        page1.breadcrumb_parent = page2
        page1.save()
        
        response = self.client.get(page2.url)
        self.assertEquals(response.status_code, 200)
        
        page3 = self._create_page('/tutu', 'toto', 'toto')
        page3.breadcrumb_text = 'Current level'
        page3.breadcrumb_parent = page2
        page3.save()
        page1.breadcrumb_parent = page3
        page1.save()

        response = self.client.get(page2.url)
        self.assertEquals(response.status_code, 200)
        
    def test_page_options(self):
        page1 = self._create_page('/toto', 'toto', 'toto')
        
        option1 = models.PageOption(name='str', value='abc', type=models.PageOption.OPTION_TYPE_STRING)
        option1.config = page1.config
        option1.save()
        
        option2 = models.PageOption(name='num', value='11', type=models.PageOption.OPTION_TYPE_NUMERIC)
        option2.config = page1.config
        option2.save()
        
        response = self.client.get(page1.url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.context['page_options']['str'], "'abc'")
        self.assertEquals(response.context['page_options']['num'], '11')
        
    def test_page_slideshow(self):
        page1 = self._create_page('/toto', 'toto', 'toto')
        
        option1 = models.PageOption(name='html_slideshow', value='1', type=models.PageOption.OPTION_TYPE_NUMERIC)
        option1.config = page1.config
        option1.save()
        
        option2 = models.PageOption(name='slideshow_effect', value='magical-effect', type=models.PageOption.OPTION_TYPE_STRING)
        option2.config = page1.config
        option2.save()
        
        response = self.client.get(page1.url)
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, option2.typed_value())
        
        option1.delete()
        response = self.client.get(page1.url)
        self.assertEquals(response.status_code, 200)
        self.assertNotContains(response, option2.typed_value())
        
    def test_page_include_header(self):
        page1 = self._create_page('/toto', 'toto', 'toto')
        
        i = models.HeaderInclude.objects.create(name = 'link')
        
        arg1 = models.HeaderIncludeArgument()
        arg1.include = i
        arg1.name = 'type'
        arg1.value = 'text/css'
        arg1.save()
        
        arg2 = models.HeaderIncludeArgument()
        arg2.include = i
        arg2.name = 'rel'
        arg2.value = '/toto.css'
        arg2.save()
        
        page1.config.header_includes.add(i)
        page1.config.save()
        
        response = self.client.get(page1.url)
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, '<link type="text/css" rel="/toto.css" />')
        
    def test_edit_mode(self):
        page = self._create_page('/toto', 'toto', 'toto')
        p1 = self._create_plugin('abc', 'def')
        p2 = self._create_plugin('123', '456')
        page.config.plugins.add(p1)
        page.config.plugins.add(p2)
        page.config.save()
        
        self.client.login(username=STAFF_USER, password=PWD)
        response = self._check_page(page)
        
        self.assertContains(response, reverse('admin:flatcms_cmspage_change', args=[page.id]))
        self.assertContains(response, reverse('admin:flatcms_pageconfig_change', args=[page.config.id]))
        self.assertContains(response, reverse('admin:flatcms_plugin_change', args=[p1.id]))
        self.assertContains(response, reverse('admin:flatcms_plugin_change', args=[p2.id]))
        self.assertContains(response, reverse('admin:index'))

        self.client.logout()
        response = self._check_page(page)

        self.assertNotContains(response, reverse('admin:flatcms_cmspage_change', args=[page.id]))
        self.assertNotContains(response, reverse('admin:flatcms_pageconfig_change', args=[page.config.id]))
        self.assertNotContains(response, reverse('admin:flatcms_plugin_change', args=[p1.id]))
        self.assertNotContains(response, reverse('admin:flatcms_plugin_change', args=[p2.id]))
        self.assertNotContains(response, reverse('admin:index'))
        
    def test_pycode(self):
        page = self._create_page('/toto', 'toto', 'toto')
        p1 = self._create_plugin('abc', 'abc#pycode#def')
        p1.py_code = settings.PYCODE_CHOICES[0][0]
        p1.save()
        page.config.plugins.add(p1)
        page.config.save()
        
        response = self._check_page(page)
        self.assertContains(response, 'abc123def')
        
    def test_pycode_error(self):
        page = self._create_page('/toto', 'toto', 'toto')
        p1 = self._create_plugin('abc', '')
        p1.py_code = settings.PYCODE_CHOICES[0][0]
        p1.save()
        page.config.plugins.add(p1)
        page.config.save()
        response = self._check_page(page)
        self.assertContains(response, _(u'pycode error'))
        
        
    def test_forbidden_pycode(self):
        page = self._create_page('/toto', 'toto', 'toto')
        p1 = self._create_plugin('abc', 'abc#pycode#def')
        p1.py_code = 'flatcms.tests.unregistred_pycode'
        p1.save()
        page.config.plugins.add(p1)
        page.config.save()
        
        response = self._check_page(page)
        self.assertContains(response, _(u'pycode error'))
        
    def test_update_links_in_page(self):
        page1 = self._create_page('/toto', 'toto', 'toto')
        old_url = page1.url
        html = u'here is a <a href="{0}">link to page</a>'.format(old_url)
        page2 = self._create_page('/page2', 'toto', html)
        page1.url = make_local('/titi')
        page1.save()
        response = self._check_page(page2)
        self.assertContains(response, page1.url)
        self.assertNotContains(response, old_url)
        
    def test_update_links_in_plugin(self):
        page1 = self._create_page('/toto', 'toto', 'toto')
        old_url = page1.url
        html = u'here is a <a href="{0}">link to page</a>'.format(old_url)
        p1 = self._create_plugin('abc', html)
        page1.config.plugins.add(p1)
        page1.config.save()
        page1.url = make_local('/titi')
        page1.save()
        response = self._check_page(page1)
        self.assertContains(response, page1.url)
        self.assertNotContains(response, old_url)
        
    def test_not_update_links_in_myself(self):
        url = make_local('/toto')
        html = u'here is a <a href="{0}">link to page</a>'.format(url)
        page1 = self._create_page(url, 'toto', html)
        page1.url = make_local('/titi')
        page1.save()
        response = self._check_page(page1)
        self.assertContains(response, page1.url)
        self.assertNotContains(response, url)
        
def dummy_pycode(request, plugin):
    if not plugin.content:
        raise Exception()
    return plugin.content.replace('#pycode#', '123')


class ImageTest(TestCase):
    def setUp(self):
        self._media_root = project_settings.MEDIA_ROOT
        project_settings.MEDIA_ROOT = '/media-unitest/'
        try:
            shutil.rmtree(project_settings.MEDIA_ROOT)
        except:
            pass

    def tearDown(self):
        shutil.rmtree(project_settings.MEDIA_ROOT)
        project_settings.MEDIA_ROOT = self._media_root

    def test_load_image(self):
        img = models.Image(name=u'test', ordering=1)
        
        filename = os.path.normpath(os.path.dirname(__file__) + '/fixtures/image.gif')
        file = open(filename)
        img.image.save(os.path.basename(filename), File(file))
        
        self.assertTrue(unicode(img))
        
    def test_load_image_unicode(self):
        img = models.Image(name=u'accentué', ordering=1)
        
        filename = os.path.normpath(os.path.dirname(__file__) + u'/fixtures/accentué.gif')
        file = open(filename)
        img.image.save(os.path.basename(filename), File(file))
        
        self.assertTrue(unicode(img))
        
class MultilingualTest(TestCase):
    def setUp(self):
        self._lang_code = getattr(project_settings, 'LANGUAGE_CODE', None)
        self._langs = getattr(project_settings, 'LANGUAGES', None)
        self._ml = getattr(project_settings, 'FLATCMS_MULTILINGUAL', None)
        
        project_settings.LANGUAGE_CODE = 'fr'
        project_settings.LANGUAGES = (('fr', u'Français'), ('en', u'English'),)
        project_settings.FLATCMS_MULTILINGUAL = True
        
        reload(settings)

    def tearDown(self):
        project_settings.LANGUAGE_CODE = self._lang_code
        project_settings.LANGUAGES = self._langs
        project_settings.FLATCMS_MULTILINGUAL = self._ml

    def _create_page(self, url, title, content, lang=None):
        if models.SiteConfig.objects.count()==0:
            site_config = models.SiteConfig()
            site_config.name = "TestConfig"
            config = models.PageConfig()
            config.name = "Default"
            config.lang = lang if lang else ''
            config.save()
            site_config.site = Site.objects.get(id=project_settings.SITE_ID)
            site_config.default_config = config
            site_config.designer = 'unittest'
            site_config.design_year = '2011'
            site_config.save()
    
        page = models.CmsPage()
        page.site = Site.objects.get(id=project_settings.SITE_ID)
        page.url = url
        page.title = title
        page.content = content
        if lang:
            config = models.PageConfig()
            config.name = "Main"
            config.lang = lang
            config.save()
            page.config = config
        page.save()
        return page
        
    def test_multilingual_page(self):
        html_tag = u'<html xmlns="http://www.w3.org/1999/xhtml" lang="{0}" xml:lang="{0}">'
        fr_page = self._create_page('/fr/toto', 'toto', 'toto', 'fr')
        en_page = self._create_page('/en/toto', 'toto', 'toto', 'en')
        def_page = self._create_page('/toto', 'toto', 'toto')
        
        response = self.client.get(fr_page.url, follow=True)
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, u'Réalisé par')
        self.assertNotContains(response, u'Designed by')
        self.assertContains(response, html_tag.format('fr'))
        self.assertNotContains(response, html_tag.format('en'))

        response = self.client.get(en_page.url, follow=True)
        self.assertEquals(response.status_code, 200)
        self.assertNotContains(response, u'Réalisé par')
        self.assertContains(response, u'Designed by')
        self.assertNotContains(response, html_tag.format('fr'))
        self.assertContains(response, html_tag.format('en'))
        
        response = self.client.get(def_page.url, follow=True)
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, u'Réalisé par')
        self.assertNotContains(response, u'Designed by')
        self.assertContains(response, html_tag.format('fr'))
        self.assertNotContains(response, html_tag.format('en'))
        
    def test_multilingual_page_default(self):
        en_page = self._create_page('/en/toto', 'toto', 'toto', 'en')#Cause site config default to be en
        fr_page = self._create_page('/fr/toto', 'toto', 'toto', 'fr')
        def_page = self._create_page('/toto', 'toto', 'toto')
        
        response = self.client.get(def_page.url, follow=True)
        self.assertEquals(response.status_code, 200)
        if 'localeurl' in project_settings.INSTALLED_APPS:
            self.assertContains(response, u'Réalisé par')
            self.assertNotContains(response, u'Designed by')
        else:
            self.assertNotContains(response, u'Réalisé par')
            self.assertContains(response, u'Designed by')
        
    def test_multilingual_no_lang_prefix(self):
        fr_page = self._create_page('/fr/toto', 'toto', 'toto', 'fr')
        en_page = self._create_page('/toto', 'toto', 'toto', 'en')
        
        response = self.client.get(en_page.url, follow=True)
        self.assertEquals(response.status_code, 200)
        if 'localeurl' in project_settings.INSTALLED_APPS:
            self.assertContains(response, u'Réalisé par')
            self.assertNotContains(response, u'Designed by')
        else:
            self.assertNotContains(response, u'Réalisé par')
            self.assertContains(response, u'Designed by')
        
    def test_multilingual_no_lang_in_default_config(self):
        def_page = self._create_page('/toto', 'toto', 'toto')
        fr_page = self._create_page('/fr/toto', 'toto', 'toto', 'fr')
        en_page = self._create_page('/en/toto', 'toto', 'toto', 'en')
        
        response = self.client.get(fr_page.url, follow=True)
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, u'Réalisé par')
        self.assertNotContains(response, u'Designed by')
    
        response = self.client.get(en_page.url, follow=True)
        self.assertEquals(response.status_code, 200)
        self.assertNotContains(response, u'Réalisé par')
        self.assertContains(response, u'Designed by')
        
        response = self.client.get(def_page.url, follow=True)
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, u'Réalisé par')
        self.assertNotContains(response, u'Designed by')
        
        
    def test_admin_pages(self):
        fr_page = self._create_page('/fr/toto', 'toto', 'toto', 'fr')
        response = self.client.get(reverse('admin:index'))
        self.assertEquals(response.status_code, 200)

        response = self.client.get(reverse('admin:flatcms_cmspage_change', args=(fr_page.id,)))
        self.assertEquals(response.status_code, 200)
        
    def test_media_url(self):
        fr_page = self._create_page('/fr/toto', 'toto', 'toto', 'fr')
        
        response = self.client.get(project_settings.MEDIA_URL+'flatcms/css/base.css')
        self.assertEquals(response.status_code, 200)

        response = self.client.get(project_settings.MEDIA_URL+'flatcms/img/draft-background.png')
        self.assertEquals(response.status_code, 200)
