# -*- coding: utf-8 -*-
"""
  flatcms : A simple CMS for Django
  (C)2010 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt
  This is very influenced by Django Flatpage
"""

from django.template import loader, RequestContext
from django.shortcuts import get_object_or_404
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.conf import settings as project_settings
#from django.core.xheaders import populate_xheaders
from django.utils.safestring import mark_safe
from django.core.servers.basehttp import FileWrapper
from django.contrib.auth import logout
from django.utils.translation import ugettext as _
from django.contrib.sites.models import Site
import models
from django.contrib.auth.views import redirect_to_login
from django.shortcuts import render_to_response
from django.core.urlresolvers import reverse
from django.utils.translation import get_language
from django.contrib.auth.decorators import user_passes_test
from utils import exec_plugin_pycode
import urlparse
import settings

def get_page_config_context(config_name=None, request=None, site_cfg=None):
    """return the list plugins and menus for a """
    has_slideshow = False
    if not config_name:
        site_cfg = site_cfg or models.SiteConfig.objects.get(site=Site.objects.get_current())
        plugins = site_cfg.default_config.plugins.order_by('ordering')
        menus = site_cfg.default_config.menu.order_by('ordering')
    else:
        plugins = models.Plugin.objects.filter(pageconfig__name__exact=config_name).order_by('ordering')
        menus = models.Menu.objects.filter(pageconfig__name__exact=config_name).order_by('ordering')
    
    for p in plugins:
        if p.py_code:
            p.content = exec_plugin_pycode(request, p)
        p.content = mark_safe(p.content)
        if p.slideshow: has_slideshow = True
    
    return {
        'plugins': plugins,
        'menus': menus,
        'has_slideshow': has_slideshow,
    }

def get_template_name(name):
    return (name.replace('.', '/')+'_.html')
    
def get_error_page(request, error_code, extra_context=None):
    site_cfg = models.SiteConfig.objects.get(site=Site.objects.get_current())
    t = loader.get_template('{0}.html'.format(error_code))
    context = get_page_config_context(None, request, site_cfg)
    err_text = _('Error {0}').format(error_code)
    context['title'] = err_text
    context['object'] = {'published': True}
    context['page'] = models.CustomPage(request.path, err_text, '', site_cfg.default_config)
    if extra_context:
        context.update(extra_context)
    c = RequestContext(request, context)
    response = HttpResponse(t.render(c))
    response.status_code = error_code
    return response

def get_404(request):
    return get_error_page(request, 404)
    
def get_403(request):
    return get_error_page(request, 403)

def get_cms_page(request, url=None, extra_context=None, tokens=None, middleware=False,
        template_name='', title='', force_create=False):
    """returns a page stored in the database"""
    if None==url:
        url = request.path
        
    if not url.startswith('/'):
        url = "/" + url
    
    possible_templates = [template_name] if template_name else []
    
    try:
        page = models.CmsPage.objects.get(url=url, site=Site.objects.get_current())
    except models.CmsPage.DoesNotExist:
        if force_create:
            page = models.CmsPage.objects.create(url=url, site=Site.objects.get_current(), title=title)
            page.save()
        else:
            alias = get_object_or_404(models.Alias, url=url, site=Site.objects.get_current())
            if alias.redirect:
                return HttpResponseRedirect(alias.page.full_url())
            else:
                page = alias.page
                page.url = alias.url #change the page url and give it the alias url
                page.site = alias.site
                if alias.config:
                    page.config = alias.config
                
    if not (page.published or request.user.is_staff):
        raise Http404
    
    if middleware and page.app_only:
        raise Http404

    if title:
        page.title = title
        
    if not page.config:
        site_cfg = models.SiteConfig.objects.get(site=Site.objects.get_current())
        page.config = site_cfg.default_config
    
    if page.config.template_name:
        possible_templates.append(get_template_name(page.config.template_name))
        
    #options linked to the page
    page_options = {}
    for option in page.config.pageoption_set.all():
        page_options[option.name] = option.typed_value()
    
    #load the site configuration
    if not possible_templates:
        possible_templates.append(get_template_name(settings.DEFAULT_TEMPLATE))
        
    # In case of registration required, redirect anonymous user to the login page.
    if page.registration_required and not request.user.is_authenticated():
        return redirect_to_login(url)
    
    #load the template
    t = loader.select_template(possible_templates)
    
    if tokens:
        for (k, v) in tokens.items():
            page.title = page.title.replace('#{0}#'.format(k), v)
            page.content = page.content.replace('#{0}#'.format(k), v)

    #Assume that title and content are safe
    page.title = mark_safe(page.title)
    page.content = mark_safe(page.content)
    
    context = {'page': page, 'object': page, 'page_options':page_options}
    context.update(get_page_config_context(page.config.name, request))
    if extra_context: context.update(extra_context)
    
    c = RequestContext(request, context)
    response = HttpResponse(t.render(c))
    #populate_xheaders(request, response, models.CmsPage, page.url)
    return response

def generate_css_style(request, css_selectors):
    """returns CSS file as a CSS response"""
    t = loader.get_template("flatcms/style.css")
    c = RequestContext(request, {'css_selectors': css_selectors})
    response = HttpResponse(t.render(c), mimetype='text/css')
    return response

def get_css_style(request):
    """returns the CSS stored in the database"""
    sels = models.CssSelector.objects.filter(site=Site.objects.get_current(),
                                             enabled=True)
    return generate_css_style(request, sels)

def get_tiny_mce_config(request):
    mce_lang = get_language()[:2]
    if mce_lang != 'fr':
        mce_lang = 'en'
    return render_to_response('flatcms/tinymce_config.js',
        {'app_name':'flatcms', 'mce_lang':mce_lang})

def get_css_style_for_mce(request):
    """returns the CSS stored in the database"""
    sels = models.CssSelector.objects.filter(site=Site.objects.get_current(),
                                             enabled=True, in_editor=True)
    return generate_css_style(request, sels)

def get_list_of_images(request):
    """returns the list of images"""
    js_code = u'var tinyMCEImageList = new Array(' 
    for img in models.Image.objects.all():
        js_code += u'\r\n\t["{0}", "{1}"],'.format(img.name, img.image.url)
    js_code = js_code[:-1] + u'\r\n);'
    return HttpResponse(js_code, mimetype='application/x-javascript')

def get_link_list(request, app_name):
    url = reverse('{0}_linklist.js'.format(app_name))
    return HttpResponseRedirect(url)

def get_list_of_links(request):
    """returns the list of links on the page"""
    js_code = u'var tinyMCELinkList = new Array(' 
    
    for page in models.CmsPage.objects.all():
        title = page.title.replace('"', '\\"')
        if not page.published:
            title = u'!-'+title
        js_code += u'\r\n\t["{1} [{0}]", "{1}"],'.format(title, page.url)
    
    for i, alias in enumerate(models.Alias.objects.all()):
        if i==0: js_code += u'\r\n\t["---", ""],'
        title = alias.page.title.replace('"', '\\"')
        if not alias.page.published:
            title = u'!-'+title
        js_code += u'\r\n\t["{1} [{0}]", "{1}"],'.format(title, alias.url)
        
    for i, dwnl in enumerate(models.Download.objects.all()):
        if i==0: js_code += u'\r\n\t["---", ""],'
        title = dwnl.file.name.replace('"', '\\"')
        js_code += u'\r\n\t["{0}", "{1}"],'.format(title, dwnl.file.url)
        
    js_code = js_code[:-1] + u'\r\n);'
    return HttpResponse(js_code, mimetype='application/x-javascript')

def get_robots_txt(request):
    """
    User-agent: *
    Disallow: /spam/egg.html
    Disallow: /egg/spam.html
    """
    text = "User-agent: *\r\n"
    disallow = (project_settings.MEDIA_URL, "/admin/") + settings.ROBOTS_DISALLOW
    for dd in disallow:
        text += "Disallow: %s\r\n" % dd
    return HttpResponse(text, mimetype='text/plain')

def user_agent(request):
    """Display the user_agent for debug purpose"""
    return HttpResponse(request.META["HTTP_USER_AGENT"])

#def serve_file(request, filename):
#    path = posixpath.normpath(urllib.unquote(filename))
#    safe_path = ''
#    for item in path.split('/'):
#        if not item: continue #remove empty section
#        drive, item = os.path.splitdrive(item)
#        head, item = os.path.split(item)
#        if item in (os.curdir, os.pardir): continue #remove . and ..
#        safe_path = os.path.join(safe_path, item).replace('\\', '/')
#    try:
#        f = file(safe_path)
#    except Exception, e:
#        return page_not_found(request)
#    try:
#        wrapper = FileWrapper(f)
#        response = HttpResponse(wrapper, mimetype=mimetypes.guess_type(safe_path)[0])
#        response['Content-Length'] = os.path.getsize(f)
#        return response
#    except Exception, e:
#        return page_not_found(request, template_name='.html')

def logout_user(request):
    """logout of the current user"""
    logout(request)
    if not hasattr(project_settings, 'LOGOUT_URL'):
        logout_url = '/'
        try:
            return HttpResponseRedirect(request.META["HTTP_REFERER"])
        except:    
            pass
    else:
        logout_url = project_settings.LOGOUT_URL
    return HttpResponseRedirect(logout_url)

@user_passes_test(lambda u: u.is_superuser)
def server_error_test(request):
    """Simulate a server error"""
    raise Exception(_("This is not a real error!"))
    
def redirect_to_homepage(request):
    return HttpResponseRedirect("/")

def redirect_to_translation(request, lang):
    referer = request.META.get("HTTP_REFERER", u'')
    if referer:
        referer = urlparse.urlparse(referer)[2]
    else:
        referer = "/"
    
    #Extract the locale part of the url
    try:
        root, locale, url = referer.split(u'/', 2)
    except ValueError:
        #No locale defined in the url --> redirect to referer
        return HttpResponseRedirect(referer)
    
    #Change the locale in the requested url
    translation_path = u'/'.join((root, lang, url))
    
    return HttpResponseRedirect(translation_path)
