# -*- coding: utf-8 -*-
"""
  tortue-logo : draw with logo into your browser
  (C)2011 - Apidev - http://www.apidev.fr
  This is distributed under GNU LGPL license, see license.txt
"""

from django.conf import settings
from django.utils.translation import ugettext as _
from django.shortcuts import get_object_or_404, render_to_response
from django.http import HttpResponse, HttpResponseRedirect
import forms
from flatcms.views import get_cms_page
from django.contrib import messages
from django.template.context import RequestContext
from models import Function
import json
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.utils.translation import get_language
from django.core.mail import mail_admins
from flatcms.views import logout_user

def save_function(request, lang):
    if request.method == 'POST' and request.is_ajax:
        form = forms.SaveFunctionForm(request.POST)
        if form.is_valid():
            name = form.cleaned_data['name']
            src_code = form.cleaned_data['src_code']
            
            if request.user and request.user.is_authenticated():
                try:
                    fct = Function.objects.get(owner=request.user, name=name)
                    fct.src_code = src_code
                    fct.lang = lang
                    fct.save()
                    message_class, message_text = u'success', _(u'{0} has been updated').format(name)
                except Function.DoesNotExist:
                    limit_number = getattr(settings, 'LOGOWEB_MAX_NB_OF_FUNCTIONS', 100)
                    if Function.objects.filter(owner=request.user).count() >= limit_number:
                        message_class, message_text = u'warning', _(u"Impossible to save {0}. You've reached the limit of {1} functions.").format(
                            name, limit_number)
                    else:
                        fct = Function.objects.create(owner=request.user, name=name, src_code=src_code, lang=lang)
                        fct.save()
                        message_class, message_text = u'success', _(u'{0} has been created').format(name)
            else:
                message_class, message_text = u'warning', _(u'You must be logged')
        else:
            message_class, message_text = u'warning', _(u'{0}'.format(form.errors()))
        
        json_result = json.dumps({'text': message_text, 'css_class': message_class})
        return HttpResponse(json_result, mimetype='application/json')
    
    raise PermissionDenied
 

def load_function(request, lang):
    if request.method == 'POST' and request.is_ajax:
        form = forms.LoadFunctionForm(request.POST)
        if form.is_valid():
            name = form.cleaned_data['name']
            src_code = u''
            if request.user and request.user.is_authenticated():
                try:
                    fct = Function.objects.get(owner=request.user, name=name, lang=lang)
                    src_code = fct.src_code
                    error_text = u''
                except Function.DoesNotExist:
                    error_text = _(u"{0} doesn't exist").format(name)
            else:
                error_text = _(u'You must be logged')
        else:
            error_text = _(u'{0}'.format(form.errors[0]))
        
        json_result = json.dumps({'error': error_text, 'src_code': src_code})
        return HttpResponse(json_result, mimetype='application/json')
    
    raise PermissionDenied

def view_profile(request):
    if request.user.is_authenticated():
        functions = Function.objects.filter(owner=request.user)
        return render_to_response('logoweb/view_profile.html', locals(),
            context_instance=RequestContext(request))
    raise PermissionDenied

def edit_profile(request):
    if request.user.is_authenticated():
        if request.method == 'POST':
            form = forms.UserProfileForm(request.POST)
            if form.is_valid():
                request.user.last_name = form.cleaned_data["lastname"]
                request.user.first_name = form.cleaned_data["firstname"]
                request.user.email = form.cleaned_data["email"]
                request.user.save()
                return HttpResponseRedirect(reverse('logoweb.views.view_profile'))
        else:
            init = {
                'lastname': request.user.last_name,
                'firstname': request.user.first_name,
                'email': request.user.email,
            }
            form = forms.UserProfileForm(initial=init)
        return render_to_response('logoweb/edit_profile.html', locals(),
            context_instance=RequestContext(request))
    raise PermissionDenied

def view_functions(request):
    if request.user.is_authenticated():
        functions = Function.objects.filter(owner=request.user)
        return render_to_response('logoweb/view_functions.html', locals(),
            context_instance=RequestContext(request))
    raise PermissionDenied
 
def delete_account(request):
    if request.user.is_authenticated():
        if request.method == "POST":
            form = forms.DeleteAccountForm(request.POST)
            if form.is_valid():
                if form.cleaned_data["confirm"]:
                    user = request.user
                    user.is_active = False
                    user.save()
                    return logout_user(request)
        else:
            form = forms.DeleteAccountForm()
        return render_to_response('logoweb/delete_account.html', locals(),
            context_instance=RequestContext(request))
    raise PermissionDenied

def update_function(request, id):
    if request.user.is_authenticated():
        fct = get_object_or_404(Function, id=id)
        if request.method == "POST":
            form = forms.UpdateFunctionForm(request.POST)
            if form.is_valid():
                if form.cleaned_data["delete"]:
                    fct.delete()
                else:
                    fct.src_code = form.cleaned_data["src_code"]
                    fct.description = form.cleaned_data["description"]
                    fct.save()
                return HttpResponseRedirect(reverse('logoweb.views.view_functions'))
        else:
            form = forms.UpdateFunctionForm(
                initial={'description': fct.description, 'src_code': fct.src_code}
            )
        return render_to_response('logoweb/update_function.html', locals(),
            context_instance=RequestContext(request))
    raise PermissionDenied

def my_functions(request):
    lang = get_language()
    if request.user.is_authenticated():
        functions = Function.objects.filter(owner=request.user)
        if functions:
            return render_to_response('logoweb/my_functions.html', locals(),
                context_instance=RequestContext(request))
        else:
            return HttpResponseRedirect('/{0}/logoweb/create-functions'.format(lang))
    else:
        return HttpResponseRedirect('/{0}/logoweb/create-account'.format(lang))
        
def feedback_form_view(request):
    if request.method == 'POST':
        form = forms.FeedbackForm(request.POST)
        if form.is_valid():
            
            #Keep in db
            feedback = form.save(commit=False)
            feedback.ip = request.META.get('REMOTE_ADDR', '')
            feedback.save()

            #Send a mail
            try:
                body = feedback.text + _(u'\n\nGo and see on site: {0}').format(
                    reverse('admin:logoweb_feedback_change', args=[feedback.id])
                )
                mail_admins(_(u'New feedback'), body)
            except Exception, msg:
                if settings.DEBUG:
                    print msg
                messages.error(request, _(u"An error occured. Please retry later"))
            else:
                return HttpResponseRedirect('/thanks-for-feedback')
    else:
        initial = None
        if request.user.is_authenticated():
            initial = {'name': request.user.username, 'email': request.user.email}
        form = forms.FeedbackForm(initial=initial)
    
    return get_cms_page(request, template_name='logoweb/feedback_form.html',
                        extra_context={'form': form})